import os
import platform
import site
import sys

import click
from requests.exceptions import HTTPError

from _incydr_cli import console
from _incydr_cli import logging_options
from _incydr_cli.cmds.actors import actors
from _incydr_cli.cmds.agents import agents
from _incydr_cli.cmds.alert_rules import alert_rules
from _incydr_cli.cmds.alerts import alerts
from _incydr_cli.cmds.audit_log import audit_log
from _incydr_cli.cmds.cases import cases
from _incydr_cli.cmds.departments import departments
from _incydr_cli.cmds.devices import devices
from _incydr_cli.cmds.directory_groups import directory_groups
from _incydr_cli.cmds.file_events import file_events
from _incydr_cli.cmds.files import files as files_client
from _incydr_cli.cmds.legal_hold import legal_hold
from _incydr_cli.cmds.orgs import orgs
from _incydr_cli.cmds.risk_profiles import risk_profiles
from _incydr_cli.cmds.sessions import sessions
from _incydr_cli.cmds.trusted_activities import trusted_activities
from _incydr_cli.cmds.users import users
from _incydr_cli.cmds.watchlists import watchlists
from _incydr_cli.core import ExceptionHandlingGroup
from _incydr_sdk.__version__ import __version__

if platform.system() in ("Darwin", "Linux"):
    os.environ["MANPAGER"] = "less -S"
else:
    ...
    # TODO: figure out Windows pager to use


@click.group(
    cls=ExceptionHandlingGroup,
    invoke_without_command=True,
    no_args_is_help=True,
    help=f"Incydr CLI by Code42 Software. Version {__version__}",
)
@click.option("--version", is_flag=True)
@click.option(
    "--python",
    is_flag=True,
    help="Print path to the python interpreter env that `incydr` is installed in.",
)
@click.option(
    "--script-dir",
    is_flag=True,
    help="Print the directory the `incydr` script was installed in (for adding to your PATH if needed).",
)
@logging_options
def incydr(version, python, script_dir):
    # Configure SDK settings
    os.environ["INCYDR_USER_AGENT_PREFIX"] = "incydrCLI (Code42; code42.com) "

    if version:
        console.print(__version__, highlight=False)
    if python:
        console.print(sys.executable, highlight=False)
        sys.exit(0)
    if script_dir:
        for root, _dirs, files in os.walk(site.PREFIXES[0]):
            if "incydr" in files or "incydr.exe" in files:
                console.print(root, highlight=False)
                sys.exit(0)

        for root, _dirs, files in os.walk(site.USER_BASE):
            if "incydr" in files or "incydr.exe" in files:
                console.print(root, highlight=False)
                sys.exit(0)


incydr.add_command(actors)
incydr.add_command(agents)
incydr.add_command(alerts)
incydr.add_command(alert_rules)
incydr.add_command(audit_log)
incydr.add_command(departments)
incydr.add_command(devices)
incydr.add_command(directory_groups)
incydr.add_command(file_events)
incydr.add_command(files_client)
incydr.add_command(cases)
incydr.add_command(risk_profiles)
incydr.add_command(sessions)
incydr.add_command(trusted_activities)
incydr.add_command(users)
incydr.add_command(orgs)
incydr.add_command(legal_hold)
incydr.add_command(watchlists)

if __name__ == "__main__":
    try:
        incydr()
    except HTTPError as err:
        console.print(f"[red]Error:[/red] {err.response.text}")



import json
from typing import Optional

import click
from click import echo

from _incydr_cli import console
from _incydr_cli import logging_options
from _incydr_cli.cmds.options.output_options import single_format_option
from _incydr_cli.cmds.options.output_options import SingleFormat
from _incydr_cli.core import IncydrCommand
from _incydr_cli.core import IncydrGroup
from _incydr_sdk.core.client import Client
from _incydr_sdk.utils import list_as_panel


@click.group(cls=IncydrGroup)
@logging_options
def departments():
    """View departments."""


@departments.command("list", cls=IncydrCommand)
@single_format_option
@click.option(
    "--name", default=None, help="Filter by departments with a matching name."
)
@logging_options
def list_(
    format_: SingleFormat,
    name: Optional[str],
):
    """
    Retrieve departments that have been pushed to Code42 from SCIM or User Directory Sync.

    The results can then be used with the watchlists commands to automatically assign users to watchlists by department.
    """
    client = Client()
    if not client.settings.use_rich and format_ == SingleFormat.rich:
        format_ = SingleFormat.json_lines
    deps = list(client.departments.v1.iter_all(name=name))

    if not deps:
        echo("No results found.")
        return

    if format_ == SingleFormat.rich:
        console.print(list_as_panel(deps, expand=False, title="Departments"))
    elif format_ == SingleFormat.json_pretty:
        console.print_json(data=deps)
    else:
        click.echo(json.dumps(deps))



import logging
import re
import sys
import warnings
from io import IOBase
from pathlib import Path
from textwrap import indent
from typing import Union

from pydantic import BaseSettings
from pydantic import Field
from pydantic import root_validator
from pydantic import SecretStr
from pydantic import validator
from requests_toolbelt.utils.dump import dump_response
from rich import pretty
from rich.console import Console
from rich.logging import RichHandler

from _incydr_sdk.enums import _Enum

# capture default displayhook so we can "uninstall" rich
_sys_displayhook = sys.displayhook
_incydr_console = Console(stderr=True)


_log_level_map = {"ERROR": 40, "WARNING": 30, "WARN": 30, "INFO": 20, "DEBUG": 10}


class LogLevel(_Enum):
    ERROR = "ERROR"
    WARNING = "WARNING"
    WARN = "WARN"
    INFO = "INFO"
    DEBUG = "DEBUG"


_std_log_formatter = logging.Formatter(
    fmt="%(asctime)s - %(name)s:%(levelname)s - %(message)s", datefmt="[%x %X]"
)
_rich_log_formatter = logging.Formatter(fmt="%(message)s", datefmt="[%x %X]")

_auth_header_regex = re.compile(r"Authorization: (Bearer|Basic) \S+")


class IncydrSettings(BaseSettings):
    """
    Configure settings on the `incydr.Client`.

    Usage:

        >> import incydr
        >>> client = incydr.Client()
        >>> client.settings.page_size = 10

    Settings can also be loaded from shell environment variables or .env files. Just prefix a setting's attribute name
    with `INCYDR_` when configuring via enviroment vars.

    For example: To load `client.settings.api_client_secret` from the environment, set
    `INCYDR_API_CLIENT_SECRET="my_secret"`.

    The `incydr.Client` loads settings in the following priority:

    - Args passed to `Client` constructor
    - Shell environment variables
    - An .env file in the current working directory
    - An .env file in `~/.config/incydr` directory

    **Attributes**:

    * **api_client_id**: `str` The API Client Identifier for authentication. env_var=`INCYDR_API_CLIENT_ID`
    * **api_client_secret**: `str` The API Client Secret for authentication. env_var=`INCYDR_API_CLIENT_SECRET`
    * **url**: `str` The URL of your Code42 API gateway instance. env_var=`INCYDR_URL`
    * **page_size**: `int` The default page size for all paginated requests. Defaults to 100. env_var=`INCYDR_PAGE_SIZE`
    * **max_response_history**: `int` The maximum number of responses the `incydr.Client.response_history` list will
        store. Defaults to 5. env_var=`INCYDR_MAX_RESPONSE_HISTORY`
    * **log_stderr**: `bool` Enables logging to stderr. Defaults to True. env_var=`INCYDR_LOG_STDERR`
    * **log_file**: `str` The file path or file-like object to write log output to. Defaults to None. env_var=`INCYDR_LOG_FILE`
    * **log_level**: `int` The level for logging messages. Defaults to `logging.WARNING`. env_var=`INCYDR_LOG_LEVEL`
    * **logger**: `logging.Logger` The logger used for client logging. Cannot be defined via environment variable. Can
        be replaced with a custom `logging.Logger`. If a custom `Logger` is supplied, the other log settings will have
        no effect and must be configured manually on the custom logger.
    * **user_agent_prefix**: `str` Prefixes all `User-Agent` headers with the supplied string.
    * **use_rich**: `bool` Enables [rich](https://rich.readthedocs.io/en/stable/introduction.html) support in logging
        and the Python repl. Defaults to True. env_var=`INCYDR_USE_RICH`
    """

    api_client_id: str = Field(env="incydr_api_client_id")
    api_client_secret: SecretStr = Field(env="incydr_api_client_secret")
    url: str = Field(env="incydr_url")
    page_size: int = Field(default=100, env="incydr_page_size")
    max_response_history: int = Field(default=5, env="incydr_max_response_history")
    use_rich: bool = Field(default=True, env="incydr_use_rich")
    log_stderr: bool = Field(default=True, env="incydr_log_stderr")
    log_file: Union[str, Path, IOBase] = Field(default=None, env="incydr_log_file")
    log_level: Union[int, str] = Field(
        default=logging.WARNING,
        env="incydr_log_level",
    )
    logger: logging.Logger = None
    user_agent_prefix: str = Field(default=None, env="incydr_user_agent_prefix")

    def __init__(self, **kwargs):
        # clear any keys from kwargs that are passed as None, which forces lookup of values
        kwargs = {k: v for k, v in kwargs.items() if v is not None}

        # if .env file in CWD, use it instead of ~/.config/incydr/.env
        if Path(".env").exists():
            kwargs["_env_file"] = ".env"
        super().__init__(**kwargs)

    class Config:
        env_file = str(Path.home() / ".config" / "incydr" / ".env")
        validate_assignment = True
        custom_logger = False

    @validator("log_level", pre=True, always=True)
    def _validate_log_level(cls, value, **kwargs):  # noqa
        try:
            return int(value)
        except ValueError:
            value = value.upper()
            LogLevel(value)
            return _log_level_map[value]

    @validator("log_file")
    def _validate_log_file(cls, value, **kwargs):  # noqa
        if isinstance(value, (str, Path)):
            p = Path(value)
            # existing file OK
            if p.exists() and p.is_file():
                value = str(p.absolute())
            # new file in existing dir OK
            elif not p.exists() and p.parent.is_dir():
                value = str(p.absolute())
            else:
                raise ValueError(f"{value} is not a valid file path for logging.")
        return value

    @validator("use_rich")
    def _validate_use_rich(cls, value, **kwargs):  # noqa
        if value:
            pretty.install()
        else:
            sys.displayhook = _sys_displayhook
        return value

    @validator("logger")
    def _validate_logger(cls, value, **kwargs):  # noqa
        if value is None:
            logger = logging.getLogger("incydr")
            # flag the logger we create with a custom attribute so we can detect user-provided loggers later
            logger._incydr = True
            return logger
        if isinstance(value, logging.Logger):
            return value
        else:
            raise ValueError(f"{value} is not a `logging.Logger`.")

    @root_validator(skip_on_failure=True)
    def _configure_logging(cls, values):  # noqa
        use_rich = values["use_rich"]
        log_file = values["log_file"]
        log_stderr = values["log_stderr"]
        log_level = values["log_level"]
        logger = values["logger"]

        if not hasattr(logger, "_incydr"):
            warnings.warn(
                "A custom logger has been set, all other log-related settings on the `incydr.Client` are ignored for custom loggers.",
                stacklevel=2,
            )
            return values

        logger.handlers.clear()

        if log_stderr and use_rich:
            rich_handler = RichHandler(console=_incydr_console, rich_tracebacks=True)
            rich_handler.setFormatter(_rich_log_formatter)
            logger.addHandler(rich_handler)

        elif log_stderr and not use_rich:
            std_handler = logging.StreamHandler()
            std_handler.setFormatter(_std_log_formatter)
            logger.addHandler(std_handler)

        if log_file and use_rich:
            if isinstance(log_file, str):
                log_file = open(log_file, "a", encoding="utf-8")
            console = Console(file=log_file, no_color=True, width=200)
            rich_file_handler = RichHandler(console=console, rich_tracebacks=True)
            rich_file_handler.setFormatter(_rich_log_formatter)
            logger.addHandler(rich_file_handler)

        elif log_file and not use_rich:
            if isinstance(log_file, str):
                std_file_handler = logging.FileHandler(
                    filename=log_file, encoding="utf-8"
                )
            else:
                std_file_handler = logging.StreamHandler(stream=log_file)
            std_file_handler.setFormatter(_std_log_formatter)
            logger.addHandler(std_file_handler)

        logger.setLevel(log_level)
        values["logger"] = logger
        return values

    def _log_response_info(self, response):
        self.logger.info(
            f"{response.request.method} {response.request.url} status_code={response.status_code}"
        )

    def _log_response_debug(self, response):
        try:
            dumped = dump_response(
                response, request_prefix=b"", response_prefix=b""
            ).decode("utf-8")
            dumped = re.sub(
                _auth_header_regex,
                "Authorization: <token_redacted>",
                dumped,
            )
            if not self.use_rich:
                dumped = indent(dumped, prefix="\t")
            self.logger.debug(dumped)
        except Exception as err:
            if isinstance(response.content, bytes):
                self.logger.debug("Unable to log binary response data.")
                self.logger.debug(response)
            else:
                self.logger.debug(f"Error dumping request/response info: {err}")
                self.logger.debug(response)

    def _log_error(self, err, invocation_str=None):
        message = str(err) if err else None
        if invocation_str:
            message = f"Exception occurred from input: '{invocation_str}'.\n" + message
        if message:
            self.logger.error(message)

    def _log_verbose_error(self, invocation_str=None, http_request=None):
        """For logging traces, invocation strs, and request parameters during exceptions to the
        error log file."""
        prefix = (
            "Exception occurred."
            if not invocation_str
            else f"Exception occurred from input: '{invocation_str}'."
        )
        self.logger.exception(f"{prefix}. See error below.")
        if http_request:
            self._log_error(f"Request parameters: {http_request.body}")



from typing import List
from typing import Optional

from pydantic import Field

from _incydr_sdk.core.models import ResponseModel


class DirectoryGroup(ResponseModel):
    group_id: Optional[str] = Field(
        None, description="A unique group ID.", example="23", alias="groupId"
    )
    name: Optional[str] = Field(None, example="Research and development")


class DirectoryGroupsPage(ResponseModel):
    """
    A model representing a list of directory groups.

    **Fields**:

    * **directory_groups**: `List[DirectoryGroup`] - The list of `n` directory groups retrieved by the query, where `n=page_size`.
    * **total_count**: `int` - Total count of directory groups retrieved by the query.
    """

    directory_groups: Optional[List[DirectoryGroup]] = Field(
        None, alias="directoryGroups"
    )
    total_count: Optional[int] = Field(
        None,
        description="The total count of all directory groups.",
        example=10,
        alias="totalCount",
    )



from datetime import datetime
from typing import List
from typing import Optional
from typing import Union

from pydantic import Field

from _incydr_sdk.core.models import Model
from _incydr_sdk.core.models import ResponseModel
from _incydr_sdk.enums.file_events import ReportType


class UserJustification(Model):
    reason: Optional[str] = Field(
        None, title="User-select justification for temporarily allowing this action."
    )
    text: Optional[str] = Field(
        None,
        title="User-select justification for temporarily allowing this action. Only applies when reason is 'Other'.",
    )


class ResponseControls(Model):
    preventative_control: Optional[str] = Field(
        None,
        alias="preventativeControl",
        example="BLOCKED",
        title="The preventative action applied to this event",
    )
    user_justification: Optional[UserJustification] = Field(
        None, alias="userJustification"
    )


class AcquiredFromGit(Model):
    repository_email: Optional[str] = Field(
        None,
        alias="repositoryEmail",
        title="The email address specified by the user who performed the Git event. This is a user-defined value and may differ from the credentials used to sign in to Git.",
    )
    repository_uri: Optional[str] = Field(
        None,
        alias="repositoryUri",
        title="Uniform Resource Identifier (URI) for the Git repository.",
    )
    repository_user: Optional[str] = Field(
        None,
        alias="repositoryUser",
        title="The username specified by the user who performed the Git event. This is a user-defined value and may differ from the credentials used to sign in to Git.",
    )


class AcquiredFromSourceUser(Model):
    email: Optional[List[str]] = Field(
        None,
        example=["first.last@example.com", "first_last_example_com"],
        title="For endpoint events where a file in cloud storage is synced to a device, the email address of the user logged in to the cloud storage provider.",
    )


class UntrustedValues(Model):
    account_names: List[str] = Field(
        ...,
        alias="accountNames",
        title="Account names that do not match an entry in your list of Trusted activity. Values are obtained from the account name metadata for the event. Only applies to event types that are evaluated for trust.",
    )
    domains: List[str] = Field(
        ...,
        title="Domains that do not match an entry in your list of Trusted activity. Values are obtained from the domain section of related metadata for the event. Only applies to event types that are evaluated for trust.",
    )
    git_repository_uris: List[str] = Field(
        ...,
        alias="gitRepositoryUris",
        title="Git URIs that do not match an entry in your list of Trusted activity. Values are obtained from the Git URI metadata for the event. Only applies to event types that are evaluated for trust.",
    )
    slack_workspaces: List[str] = Field(
        ...,
        alias="slackWorkspaces",
        title="Slack workspaces that do not match an entry in your list of Trusted activity. Values are obtained from the Slack metadata for the event. Only applies to event types that are evaluated for trust.",
    )
    url_paths: List[str] = Field(
        ...,
        alias="urlPaths",
        title="URL paths that do not match an entry in your list of Trusted activity. Values are obtained from the URL metadata for the event. Only applies to event types that are evaluated for trust.",
    )


class DestinationEmail(Model):
    recipients: Optional[List[str]] = Field(
        description="The email addresses of those who received the email. Includes the To, Cc, and Bcc recipients.",
        example=["cody@example.com", "theboss@example.com"],
    )
    subject: Optional[str] = Field(
        description="The subject of the email message.",
        example="Important business documents",
    )


class DestinationUser(Model):
    email: Optional[List[str]] = Field(
        description="For endpoint events where a file in cloud storage is synced to a device, the email address of the user logged in to the cloud storage provider. For cloud events, the email addresses of users added as sharing recipients. In some case, OneDrive events may return multiple values, but this is often the same username formatted in different ways.",
        example=["first.last@example.com", "first_last_example_com"],
    )


class FileClassification(Model):
    value: Optional[str] = Field(
        description="The classification value applied to the file.",
        example="Classified",
    )
    vendor: Optional[str] = Field(
        description="The name of the vendor that classified the file.",
        example="MICROSOFT INFORMATION PROTECTION",
    )


class Hash(Model):
    md5: Optional[str] = Field(
        description="The MD5 hash of the file contents.",
        example="a162591e78eb2c816a28907d3ac020f9",
    )
    md5_error: Optional[str] = Field(
        alias="md5Error", description="Reason the MD5 hash is unavailable."
    )
    sha256: Optional[str] = Field(
        description="The SHA-256 hash of the file contents.",
        example="ded96d69c63754472efc4aa86fed68d4e17784b38089851cfa84e699e48b4155",
    )
    sha256_error: Optional[str] = Field(
        alias="sha256Error", description="Reason the SHA-256 hash is unavailable."
    )


class Extension(Model):
    browser: Optional[str] = Field(
        None, title="The web browser in which the event occurred."
    )
    version: Optional[str] = Field(
        None,
        title="The version of the Code42 Incydr extension installed when the event occurred.",
    )
    logged_in_user: Optional[str] = Field(
        None,
        alias="loggedInUser",
        title="The user signed in to the active tab where the event occurred. For example, the user signed in to Gmail. This may differ from the user account signed in to the browser itself.",
    )


class Process(Model):
    executable: Optional[str] = Field(
        None,
        description="The name of the process that accessed the file, as reported by the device’s operating system. Depending on your Code42 product plan, this value may be null for some event types.",
        example="bash",
    )
    owner: Optional[str] = Field(
        None,
        description="The username of the process owner, as reported by the device’s operating system. Depending on your Code42 product plan, this value may be null for some event types.",
        example="root",
    )
    extension: Optional[Extension]


class RemovableMedia(Model):
    bus_type: Optional[str] = Field(
        alias="busType",
        description="For events detected on removable media, indicates the communication system used to transfer data between the host and the removable device.",
        example="USB 3.0 Bus",
    )
    capacity: Optional[int] = Field(
        description="For events detected on removable media, the capacity of the removable device in bytes.",
        example=15631122432,
    )
    media_name: Optional[str] = Field(
        alias="mediaName",
        description="For events detected on removable media, the media name of the device, as reported by the vendor/device. This is usually very similar to the productName, but can vary based on the type of device. For example, if the device is a hard drive in a USB enclosure, this may be the combination of the drive model and the enclosure model.\nThis value is not provided by all devices, so it may be null in some cases.",
        example="Cruzer Blade",
    )
    name: Optional[str] = Field(
        description="For events detected on removable media, the name of the removable device.",
        example="JUMPDRIVE",
    )
    partition_id: Optional[List[str]] = Field(
        alias="partitionId",
        description="For events detected on removable media, a unique identifier assigned to the volume/partition when it was formatted. Windows devices refer to this as the VolumeGuid. On Mac devices, this is the Disk / Partition UUID, which appears when running the Terminal command diskUtil info.",
        example=["disk0s2", "disk0s3"],
    )
    serial_number: Optional[str] = Field(
        alias="serialNumber",
        description="For events detected on removable media, the serial number of the removable device.",
        example="4C531001550407108465",
    )
    vendor: Optional[str] = Field(
        description="For events detected on removable media, the vendor of the removable device.",
        example="SanDisk",
    )
    volume_name: Optional[List[str]] = Field(
        alias="volumeName",
        description='For events detected on removable media, the name assigned to the volume when it was formatted, as reported by the device\'s operating system. This is also frequently called the "partition" name.',
        example=["MY_FILES"],
    )


class Report(Model):
    count: Optional[int] = Field(
        description="The total number of rows returned in the report.", example=20
    )
    description: Optional[str] = Field(
        description="The description of the report.",
        example="Top 20 accounts based on annual revenue",
    )
    headers: Optional[List[str]] = Field(
        description="The list of column headers that are in the report.",
        example=[
            "USERNAME",
            "ACCOUNT_NAME",
            "TYPE",
            "DUE_DATE",
            "LAST_UPDATE",
            "ADDRESS1_STATE",
        ],
    )
    id: Optional[str] = Field(
        description="The ID of the report associated with this event.",
        example="00OB00000042FHdMAM",
    )
    name: Optional[str] = Field(
        description="The display name of the report.",
        example="Top Accounts Report",
    )
    type: Optional[Union[ReportType, str]] = Field(
        description='Indicates if the report is "REPORT_TYPE_AD_HOC" or "REPORT_TYPE_SAVED".',
        example="REPORT_TYPE_SAVED",
    )


class RiskIndicator(Model):
    name: Optional[str] = Field(
        description="Name of the risk indicator.",
        example="Browser upload",
    )
    id: Optional[str] = Field(
        None, title="The unique identifier for the risk indicator."
    )
    weight: Optional[int] = Field(
        description="Configured weight of the risk indicator at the time this event was seen.",
        example=5,
    )


class SourceEmail(Model):
    from_: Optional[str] = Field(
        alias="from",
        description='The display name of the sender, as it appears in the "From" field in the email. In many cases, this is the same as source.email.sender, but it can be different if the message is sent by a server or other mail agent on behalf of someone else.',
        example="ari@example.com",
    )
    sender: Optional[str] = Field(
        description="The address of the entity responsible for transmitting the message. In many cases, this is the same as source.email.from, but it can be different if the message is sent by a server or other mail agent on behalf of someone else.",
        example="ari@example.com",
    )


class SourceUser(Model):
    email: Optional[List[str]] = Field(
        None,
        description="For endpoint events where a file in cloud storage is synced to a device, the email address of the user logged in to the cloud storage provider.",
        example=["first.last@example.com", "first_last_example_com"],
    )


class Tab(Model):
    title: Optional[str] = Field(
        description="The title of this app or browser tab.",
        example="Example Domain",
    )
    title_error: Optional[str] = Field(
        alias="titleError",
        description="Reason the title of this app or browser tab is unavailable.",
        example="InsufficientPermissions",
    )
    url: Optional[str] = Field(
        description="The URL of this browser tab.",
        example="https://example.com/",
    )
    url_error: Optional[str] = Field(
        alias="urlError",
        description="Reason the URL of this browser tab is unavailable.",
        example="InsufficientPermissions",
    )


class User(Model):
    device_uid: Optional[str] = Field(
        alias="deviceUid",
        description="Unique identifier for the device. Null if the file event occurred on a cloud provider.",
        example=24681,
    )
    email: Optional[str] = Field(
        description="The Code42 username used to sign in to the Code42 app on the device. Null if the file event occurred on a cloud provider.",
        example="cody@example.com",
    )
    id: Optional[str] = Field(
        description="Unique identifier for the user of the Code42 app on the device. Null if the file event occurred on a cloud provider.",
        example=1138,
    )


class AcquiredFrom(Model):
    event_id: Optional[str] = Field(
        None,
        alias="eventId",
        example="0_147e9445-2f30-4a91-8b2a-9455332e880a_973435567569502913_986467523038446097_163",
        title="The unique identifier for the event.",
    )
    tabs: Optional[List[Tab]] = Field(
        None, title="Metadata about the browser tab source."
    )
    source_account_name: Optional[str] = Field(
        None,
        alias="sourceAccountName",
        title="For cloud sync apps installed on user devices, the name of the cloud account where the event was observed. This can help identify if the activity occurred in a business or personal account.",
    )
    source_account_type: Optional[str] = Field(
        None,
        alias="sourceAccountType",
        title="For cloud sync apps installed on user devices, the type of account where the event was observed. For example, 'BUSINESS' or 'PERSONAL'.",
    )
    source_category: Optional[str] = Field(
        None,
        alias="sourceCategory",
        example="Social Media",
        title="General category of where the file originated. For example: Cloud Storage, Email, Social Media.",
    )
    source_name: Optional[str] = Field(
        None,
        alias="sourceName",
        example="Mari's MacBook",
        title="The name reported by the device's operating system.  This may be different than the device name in the Code42 console.",
    )
    source_user: Optional[AcquiredFromSourceUser] = Field(None, alias="sourceUser")
    agent_timestamp: Optional[datetime] = Field(
        None,
        alias="agentTimestamp",
        example="2020-10-27T15:16:05.369203Z",
        title="Date and time that the Code42 service on the device detected an event; based on the device’s system clock and reported in Coordinated Universal Time (UTC).",
    )
    user_email: Optional[str] = Field(
        None,
        alias="userEmail",
        example="cody@example.com",
        title="The Code42 username used to sign in to the Code42 app on the device (for endpoint events) or the cloud service username of the person who caused the event (for cloud events).",
    )
    event_action: Optional[str] = Field(
        None,
        alias="eventAction",
        example="file-downloaded",
        title="The type of file event observed. For example: file-modified, application-read, removable-media-created.",
    )
    source_domains: Optional[List[str]] = Field(
        None,
        alias="sourceDomains",
        example="example.com",
        title="The domain section of the URLs reported in file.acquiredFrom.tabs.url.",
    )
    file_name: Optional[str] = Field(
        None,
        alias="fileName",
        example="example.txt",
        title="The name of the file, including the file extension.",
    )
    md5: Optional[str] = Field(
        None,
        example="6123bbce7f3937667a368bbb9f3d79ce",
        title="The MD5 hash of the file contents.",
    )
    git: Optional[AcquiredFromGit] = None


class Destination(Model):
    account_name: Optional[str] = Field(
        alias="accountName",
        description="For cloud sync apps installed on user devices, the name of the cloud account where the event was observed. This can help identify if the activity occurred in a business or personal account.",
    )
    account_type: Optional[str] = Field(
        alias="accountType",
        description="For cloud sync apps installed on user devices, the type of account where the event was observed. For example, 'BUSINESS' or 'PERSONAL'.",
        example="BUSINESS",
    )
    category: Optional[str] = Field(
        description="General category of where the file originated. For example: Cloud Storage, Email, Social Media.",
        example="Social Media",
    )
    domains: Optional[List[str]] = Field(
        description="The domain section of the URLs reported in destination.tabs.url.",
    )
    email: Optional[DestinationEmail] = Field(
        description="Metadata about the destination email."
    )
    ip: Optional[str] = Field(
        description="The external IP address of the user's device.",
        example="127.0.0.1",
    )
    name: Optional[str] = Field(
        description="The name reported by the device's operating system.  This may be different than the device name in the Code42 console.",
        example="Mari's MacBook",
    )
    operating_system: Optional[str] = Field(
        alias="operatingSystem",
        description="The operating system of the destination device.",
        example="Windows 10",
    )
    print_job_name: Optional[str] = Field(
        alias="printJobName",
        description="For print events, the name of the print job, as reported by the user's device.",
        example="printer.exe",
    )
    printer_name: Optional[str] = Field(
        alias="printerName",
        description="For print events, the name of the printer the job was sent to.",
        example="OfficeJet",
    )
    printed_files_backup_path: Optional[str] = Field(alias="printedFilesBackupPath")
    private_ip: Optional[List[str]] = Field(
        alias="privateIp",
        description="The IP address of the user's device on your internal network, including Network interfaces, Virtual Network Interface controllers (NICs), and Loopback/non-routable addresses.",
        example=["127.0.0.1", "127.0.0.2"],
    )
    removable_media: Optional[RemovableMedia] = Field(
        alias="removableMedia",
        description="Metadata about the removable media destination.",
    )
    tabs: Optional[List[Tab]] = Field(
        description="Metadata about the browser tab destination.",
    )
    user: Optional[DestinationUser] = Field(
        description="Metadata about the destination user."
    )
    remote_hostname: Optional[str] = Field(
        None,
        alias="remoteHostname",
        title="For events where a file transfer tool was used, the destination hostname.",
    )


class File(Model):
    acquired_from: Optional[List[AcquiredFrom]] = Field(None, alias="acquiredFrom")
    category: Optional[str] = Field(
        description="A categorization of the file that is inferred from MIME type.",
        example="Audio",
    )
    category_by_bytes: Optional[str] = Field(
        alias="categoryByBytes",
        description="A categorization of the file based on its contents.",
        example="Image",
    )
    category_by_extension: Optional[str] = Field(
        alias="categoryByExtension",
        description="A categorization of the file based on its extension.",
        example="Document",
    )
    change_type: Optional[str] = Field(
        None,
        alias="changeType",
        description="The action that caused the event. For example: CREATED, MODIFIED, DELETED.",
        example="CREATED",
    )
    classifications: Optional[List[FileClassification]] = Field(
        description="Data provided by an external file classification vendor."
    )
    cloud_drive_id: Optional[str] = Field(
        alias="cloudDriveId",
        description="Unique identifier reported by the cloud provider for the drive containing the file at the time the event occurred.",
        example="RvBpZ48u2m",
    )
    created: Optional[datetime] = Field(
        description="File creation timestamp as reported by the device's operating system in Coordinated Universal Time (UTC); available for Mac and Windows NTFS devices only.",
        example="2020-10-27T15:16:05.369203Z",
    )
    directory: Optional[str] = Field(
        description="The file location on the user's device; a forward or backslash must be included at the end of the filepath. Possibly null if the file event occurred on a cloud provider.",
        example="/Users/alix/Documents/",
    )
    original_directory: Optional[str] = Field(
        None,
        alias="originalDirectory",
        example="/Users/alix/Documents/",
        title="The original file location on the user’s device or cloud service location; a forward or backslash must be included at the end of the filepath. Possibly null if the file event occurred on a cloud provider.",
    )
    directory_id: Optional[List[str]] = Field(
        alias="directoryId",
        description="Unique identifiers of the parent drives that contain the file; searching on directoryId will return events for all of the files contained in the parent drive.",
        example=["1234", "56d78"],
    )
    hash: Optional[Hash] = Field(description="Hash values of the file.")
    id: Optional[str] = Field(
        description="Unique identifier reported by the cloud provider for the file associated with the event.",
        example="PUL5zWLRrdudiJZ1OCWw",
    )
    mime_type: Optional[str] = Field(
        None,
        alias="mimeType",
        title="The MIME type of the file. For endpoint events, if the mimeTypeByBytes differs from mimeTypeByExtension, this indicates the most likely MIME type for the file. For activity observed by a web browser, this is the only MIME type reported.",
    )
    mime_type_by_bytes: Optional[str] = Field(
        alias="mimeTypeByBytes",
        description="The MIME type of the file based on its contents.",
        example="text/csv",
    )
    mime_type_by_extension: Optional[str] = Field(
        alias="mimeTypeByExtension",
        description="The MIME type of the file based on its extension.",
        example="audio/vorbis",
    )
    modified: Optional[datetime] = Field(
        description="File modification timestamp as reported by the device's operating system.  This only indicates changes to file contents.  Changes to file permissions, file owner, or other metadata are not reflected in this timestamp.  Date is reported in Coordinated Universal Time (UTC).",
        example="2020-10-27T15:16:05.369203Z",
    )
    name: Optional[str] = Field(
        description="The name of the file, including the file extension.",
        example="ReadMe.md",
    )
    original_name: Optional[str] = Field(
        None,
        alias="originalName",
        example="ReadMe.md",
        title="The original name of the file, including the file extension.",
    )
    owner: Optional[str] = Field(
        description="The name of the user who owns the file as reported by the device's file system.",
        example="ari.example",
    )
    size_in_bytes: Optional[int] = Field(
        alias="sizeInBytes", description="Size of the file in bytes.", example=256
    )
    url: Optional[str] = Field(
        description="URL reported by the cloud provider at the time the event occurred.",
        example="https://example.com",
    )
    archive_id: Optional[str] = Field(
        None,
        alias="archiveId",
        title="Unique identifier for files identified as an archive, such as .zip files.",
    )
    parent_archive_id: Optional[str] = Field(
        None,
        alias="parentArchiveId",
        title="For files contained within an archive (such as a .zip file), the unique identifier for that archive; searching on parentArchiveID returns events for all files contained within that archive",
    )
    password_protected: Optional[bool] = Field(
        None,
        alias="passwordProtected",
        title="Indicates if this file is password protected.",
    )


class Risk(Model):
    indicators: Optional[List[RiskIndicator]] = Field(
        description="List of risk indicators identified for this event. If more than one risk indicator applies to this event, the sum of all indicators determines the total risk score.",
    )
    score: Optional[int] = Field(
        description="Sum of the weights for each risk indicator. This score is used to determine the overall risk severity of the event.",
        example=12,
    )
    severity: Optional[str] = Field(
        description="The general risk assessment of the event, based on the numeric score.",
        example="CRITICAL",
    )
    trust_reason: Optional[str] = Field(
        alias="trustReason",
        description="The reason the event is trusted. trustReason is only populated if trusted is true for this event.",
        example="TRUSTED_DOMAIN_BROWSER_URL",
    )
    trusted: Optional[bool] = Field(
        description="Indicates whether or not the file activity is trusted based on your Data Preferences settings.",
        example=True,
    )
    untrusted_values: UntrustedValues = Field(..., alias="untrustedValues")


class Source(Model):
    account_name: Optional[str] = Field(
        None,
        alias="accountName",
        description="For cloud sync apps installed on user devices, the name of the cloud account where the event was observed. This can help identify if the activity occurred in a business or personal account.",
    )
    account_type: Optional[str] = Field(
        None,
        alias="accountType",
        description="For cloud sync apps installed on user devices, the type of account where the event was observed. For example, ‘BUSINESS’ or ‘PERSONAL’.",
    )
    category: Optional[str] = Field(
        description="General category of where the file originated. For example: Cloud Storage, Email, Social Media.",
        example="Social Media",
    )
    domain: Optional[str] = Field(
        description="Fully qualified domain name (FQDN) for the user's device at the time the event is recorded.  If the device is unable to resolve the domain name of the host, it reports the IP address of the host.",
        example="localhost",
    )
    domains: Optional[List[str]] = Field(
        description="The domain section of the URLs reported in source.tabs.url. (Note: Although similar in name, this field has no relation to source.domain, which reports the FQDN or IP address of the user’s device.)",
    )
    email: Optional[SourceEmail] = Field(description="Metadata about the email source.")
    ip: Optional[str] = Field(
        description="The external IP address of the user's device.",
        example="127.0.0.1",
    )
    name: Optional[str] = Field(
        description="The name reported by the device's operating system.  This may be different than the device name in the Code42 console.",
        example="Mari's MacBook",
    )
    operating_system: Optional[str] = Field(
        alias="operatingSystem",
        description="The operating system of the source device.",
        example="Windows 10",
    )
    private_ip: Optional[List[str]] = Field(
        alias="privateIp",
        description="The IP address of the user's device on your internal network, including Network interfaces, Virtual Network Interface controllers (NICs), and Loopback/non-routable addresses.",
        example=["127.0.0.1", "127.0.0.2"],
    )
    remote_hostname: Optional[str] = Field(
        None,
        alias="remoteHostname",
        title="For events where a file transfer tool was used, the source hostname.",
    )
    removable_media: Optional[RemovableMedia] = Field(
        alias="removableMedia",
        description="Metadata about the removable media source.",
    )
    tabs: Optional[List[Tab]] = Field(
        description="Metadata about the browser tab source."
    )
    user: Optional[SourceUser] = Field(
        None, description="Metadata about the source user."
    )


class RelatedEvent(Model):
    agent_timestamp: Optional[datetime] = Field(
        alias="agentTimestamp",
        description="Date and time that the Code42 service on the device detected an event; based on the device’s system clock and reported in Coordinated Universal Time (UTC).",
        example="2020-10-27T15:16:05.369203Z",
    )
    event_action: Optional[str] = Field(
        alias="eventAction",
        description="The type of file event observed. For example: file-modified, application-read, removable-media-created.",
        example="file-downloaded",
    )
    id: Optional[str] = Field(
        description="The unique identifier for the event.",
        example="0_147e9445-2f30-4a91-8b2a-9455332e880a_973435567569502913_986467523038446097_163",
    )
    source_category: Optional[str] = Field(
        alias="sourceCategory",
        description="General category of where the file originated. For example: Cloud Storage, Email, Social Media.",
        example="Social Media",
    )
    source_name: Optional[str] = Field(
        alias="sourceName",
        description="The name reported by the device's operating system.  This may be different than the device name in the Code42 console.",
        example="Mari's MacBook",
    )
    tabs: Optional[List[Tab]] = Field(
        description="Metadata about the browser tab source."
    )
    user_email: Optional[str] = Field(
        alias="userEmail",
        description="The Code42 username used to sign in to the Code42 app on the device (for endpoint events) or the cloud service username of the person who caused the event (for cloud events).",
        example="cody@example.com",
    )


class Event(Model):
    action: Optional[str] = Field(
        description="The type of file event observed. For example: file-modified, application-read, removable-media-created.",
        example="file-downloaded",
    )
    id: Optional[str] = Field(
        description="The unique identifier for the event.",
        example="0_147e9445-2f30-4a91-8b2a-9455332e880a_973435567569502913_986467523038446097_163",
    )
    ingested: Optional[datetime] = Field(
        description="Date and time the event was initially received by Code42; timestamp is based on the Code42 server system clock and reported in Coordinated Universal Time (UTC).",
        example="2020-10-27T15:15:05.369203Z",
    )
    inserted: Optional[datetime] = Field(
        description="Date and time the event processing is completed by Code42; timestamp is based on the Code42 server system clock and reported in Coordinated Universal Time (UTC). Typically occurs very soon after the event.ingested time.",
        example="2020-10-27T15:16:05.369203Z",
    )
    observer: Optional[str] = Field(
        description="The data source that captured the file event. For example: GoogleDrive, Office365, Salesforce.",
        example="Endpoint",
    )
    detector_display_name: Optional[str] = Field(
        None,
        alias="detectorDisplayName",
        title="Indicates the name you provided when the cloud data connection was initially configured in the Code42 console.",
    )
    related_events: Optional[List[RelatedEvent]] = Field(
        alias="relatedEvents",
        description="List of other events associated with this file. This can help determine the origin of the file.",
    )
    share_type: Optional[List[str]] = Field(
        alias="shareType",
        description="Sharing types added by this event.",
        example=["SharedViaLink"],
    )
    vector: Optional[str] = Field(
        None,
        example="GIT_PUSH",
        title="The method of file movement. For example: UPLOADED, DOWNLOADED, EMAILED.",
    )


class Git(Model):
    event_id: Optional[str] = Field(
        None,
        alias="eventId",
        title="A global unique identifier (GUID) generated by Incydr for this Git event. All files associated with this event have the same Git event ID. A single Git event can be associated with multiple file events.",
    )
    last_commit_hash: Optional[str] = Field(
        None,
        alias="lastCommitHash",
        title="Hash value from the most recent commit in this Git event.",
    )
    repository_email: Optional[str] = Field(
        None,
        alias="repositoryEmail",
        title="The email address specified by the user who performed the Git event. This is a user-defined value and may differ from the credentials used to sign in to Git.",
    )
    repository_endpoint_path: Optional[str] = Field(
        None,
        alias="repositoryEndpointPath",
        title="File path of the local Git repository on the user's endpoint.",
    )
    repository_uri: Optional[str] = Field(
        None,
        alias="repositoryUri",
        title="Uniform Resource Identifier (URI) for the Git repository.",
    )
    repository_user: Optional[str] = Field(
        None,
        alias="repositoryUser",
        title="The username specified by the user who performed the Git event. This is a user-defined value and may differ from the credentials used to sign in to Git.",
    )


class FileEventV2(ResponseModel):
    """
    New fields are often being made available on the event response and this model definition may not contain the most recent additions.
    See the response definition for the [File Event Search API](https://developer.code42.com/api/#tag/File-Events/operation/searchEvents) for the most up to date information on what data is available.

    **Fields**:

    * **timestamp**: - The date and time that the Code42 service on the device detected the event.  This timestamp is based on the device’s system clock and reported in Coordinated Universal Time (UTC).
    * **destination**: `Destination` - A [`Destination`] object containing metadata about the destination of the file event.
    * **event**: `Event` - An [`Event`] object containing summary information about the event.
    * **file**: `File` - A [`File`] object containing metadata about the file for this event.
    * **process**: `Process` - A [`Process`] object containing metadata about the process associated with the event.
    * **report**: `Report` - A [`Report`] object containing metadata for reports from 3rd party sources, such Salesforce downloads.
    * **risk**: `Risk` - A [`Risk`] object containing metadata on risk factors associated with the event.
    * **source**: `Source` - A [`Source`] object containing metadata about the source of the file event.
    * **user**: `User` - A [`User`] object containing metadata Attributes of the Code42 username signed in to the Code42 app on the device.
    * **git**: `Git` - A [`Git`] object containing git details for the event (if applicable).
    """

    timestamp: Optional[datetime] = Field(
        alias="@timestamp",
        description="Date and time that the Code42 service on the device detected an event; based on the device’s system clock and reported in Coordinated Universal Time (UTC).",
        example="2020-10-27T15:16:05.369203Z",
    )
    destination: Optional[Destination] = Field(
        description="Metadata about the destination of the file event.",
    )
    event: Optional[Event] = Field(
        description="Summary information about the event.",
    )
    file: Optional[File] = Field(
        description="Metadata about the file for this event.",
    )
    process: Optional[Process] = Field(
        description="Metadata about the process associated with the event.",
    )
    report: Optional[Report] = Field(
        description="Metadata for reports from 3rd party sources, such Salesforce downloads.",
    )
    response_controls: Optional[ResponseControls] = Field(
        alias="responseControls",
        description="Metadata about preventative actions applied to file activity. Only applies to events for users on a preventative watchlist.",
    )
    risk: Optional[Risk] = Field(
        description="Risk factor metadata.",
    )
    source: Optional[Source] = Field(
        description="Metadata about the source of the file event.",
    )
    user: Optional[User] = Field(
        description="Attributes of the the Code42 username signed in to the Code42 app on the device.",
    )
    git: Optional[Git] = Field(
        description="Git details for the event.",
    )



from itertools import count
from typing import Iterator
from typing import List
from typing import Union
from warnings import warn

from _incydr_sdk.enums.watchlists import WatchlistType
from _incydr_sdk.exceptions import WatchlistNotFoundError
from _incydr_sdk.watchlists.models.requests import CreateWatchlistRequest
from _incydr_sdk.watchlists.models.requests import ListWatchlistsRequest
from _incydr_sdk.watchlists.models.requests import UpdateExcludedUsersRequest
from _incydr_sdk.watchlists.models.requests import UpdateIncludedDepartmentsRequest
from _incydr_sdk.watchlists.models.requests import UpdateIncludedDirectoryGroupsRequest
from _incydr_sdk.watchlists.models.requests import UpdateIncludedUsersRequest
from _incydr_sdk.watchlists.models.requests import UpdateWatchlistRequest
from _incydr_sdk.watchlists.models.responses import ExcludedUsersList
from _incydr_sdk.watchlists.models.responses import IncludedDepartment
from _incydr_sdk.watchlists.models.responses import IncludedDepartmentsList
from _incydr_sdk.watchlists.models.responses import IncludedDirectoryGroup
from _incydr_sdk.watchlists.models.responses import IncludedDirectoryGroupsList
from _incydr_sdk.watchlists.models.responses import IncludedUsersList
from _incydr_sdk.watchlists.models.responses import Watchlist
from _incydr_sdk.watchlists.models.responses import WatchlistMembersList
from _incydr_sdk.watchlists.models.responses import WatchlistsPage
from _incydr_sdk.watchlists.models.responses import WatchlistUser

# This client is deprecated 2025-03. It will be removed 2026-03.


class WatchlistsV1:
    """
    Client for `/v1/watchlists` endpoints.

    This client is deprecated. Use the WatchlistsV2 client instead.

    Usage example:

        >>> import incydr
        >>> client = incydr.Client(**kwargs)
        >>> client.watchlists.v1.get_page()
    """

    def __init__(self, parent):
        self._parent = parent
        self._watchlist_type_id_map = {}
        self._uri = "/v1/watchlists"

    def get_page(
        self, page_num: int = 1, page_size: int = None, user_id: str = None
    ) -> WatchlistsPage:
        """
        Get a page of watchlists.

        Filter results by passing appropriate parameters:

        **Parameters**:

        * **page_num**: `int` - Page number for results, starting at 1.
        * **page_size**: `int` - Max number of results to return for a page.
        * **user_id**: `str` - Matches watchlists where the user is a member.

        **Returns**: A [`WatchlistsPage`][watchlistspage-model] object.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        page_size = page_size or self._parent.settings.page_size
        data = ListWatchlistsRequest(page=page_num, pageSize=page_size, userId=user_id)
        response = self._parent.session.get(self._uri, params=data.dict())
        return WatchlistsPage.parse_response(response)

    def iter_all(
        self, page_size: int = None, user_id: str = None
    ) -> Iterator[Watchlist]:
        """
        Iterate over all watchlists.

        Accepts the same parameters as `.get_page()` excepting `page_num`.

        **Returns**: A generator yielding individual [`Watchlist`][watchlist-model] objects.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        page_size = page_size or self._parent.settings.page_size
        for page_num in count(1):
            page = self.get_page(
                page_num=page_num, page_size=page_size, user_id=user_id
            )
            yield from page.watchlists
            if len(page.watchlists) < page_size:
                break

    def get(self, watchlist_id: str) -> Watchlist:
        """
        Get a single watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.

        **Returns**: A [`Watchlist`][watchlist-model] object.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        response = self._parent.session.get(f"{self._uri}/{watchlist_id}")
        return Watchlist.parse_response(response)

    def create(
        self, watchlist_type: WatchlistType, title: str = None, description: str = None
    ) -> Watchlist:
        """
        Create a new watchlist.

        **Parameters**:

        * **watchlist_type**: [`WatchlistType`][watchlist-types] (required) - Type of the watchlist to create.
        * **title**: The required title for a custom watchlist.
        * **description**: The optional description for a custom watchlist.

        **Returns**: A ['Watchlist`][watchlist-model] object.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        if watchlist_type == "CUSTOM":
            if title is None:
                raise ValueError("`title` value is required for custom watchlists.")

        data = CreateWatchlistRequest(
            description=description, title=title, watchlistType=watchlist_type
        )
        response = self._parent.session.post(url=self._uri, json=data.dict())
        watchlist = Watchlist.parse_response(response)
        self._watchlist_type_id_map[watchlist_type] = watchlist.watchlist_id
        return watchlist

    def delete(self, watchlist_id: str):
        """
        Delete a watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.

        **Returns**: A `requests.Response` indicating success.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        return self._parent.session.delete(f"{self._uri}/{watchlist_id}")

    def update(
        self, watchlist_id: str, title: str = None, description: str = None
    ) -> Watchlist:
        """
        Update a custom watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.
        * **title**: `str` - Updated title for a custom watchlist.  Defaults to None.
        * **description: `str` - Updated description for a custom watchlist.  Pass an empty string to clear this field.  Defaults to None.

        **Returns**: A [`Watchlist`][watchlist-model] object.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        paths = []
        if title:
            paths += ["title"]
        if description:
            paths += ["description"]
        query = {"paths": paths}
        data = UpdateWatchlistRequest(description=description, title=title)
        response = self._parent.session.patch(
            f"{self._uri}/{watchlist_id}", params=query, json=data.dict()
        )
        return Watchlist.parse_response(response)

    def get_member(self, watchlist_id: str, user_id: str) -> WatchlistUser:
        """
        Get a single member of a watchlist. A member may have been added as an included user, or is a member of an included department, etc.

        **Parameters**:

         * **watchlist_id**: `str` (required) - Watchlist ID.
        * **user_id**: `str` (required) - Unique user ID.

        **Returns**: A [`WatchlistUser`][watchlistuser-model] object.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        response = self._parent.session.get(
            f"{self._uri}/{watchlist_id}/members/{user_id}"
        )
        return WatchlistUser.parse_response(response)

    def list_members(
        self, watchlist_id: Union[str, WatchlistType]
    ) -> WatchlistMembersList:
        """
        Get a list of all members of a watchlist. These users may have been added as an included user, or are members of an included department, etc.

        **Parameters**:

        * **watchlist_id**: `str`(required) - Watchlist ID.

        **Returns**: A [`WatchlistMembersList`][watchlistmemberslist-model] object.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        response = self._parent.session.get(f"{self._uri}/{watchlist_id}/members")
        return WatchlistMembersList.parse_response(response)

    def add_included_users(self, watchlist_id: str, user_ids: Union[str, List[str]]):
        """
        Include individual users on a watchlist.

        **Parameters**:

        * **watchlist**: `str` (required) - Watchlist ID.
        * **user_ids**: `str`, `List[str]` (required) - List of unique user IDs to include on the watchlist. A maximum
            of 100 users can be processed in a single request.

        **Returns**: A `requests.Response` indicating success.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        data = UpdateIncludedUsersRequest(
            userIds=user_ids if isinstance(user_ids, List) else [user_ids],
            watchlistId=watchlist_id,
        )
        return self._parent.session.post(
            url=f"{self._uri}/{watchlist_id}/included-users/add", json=data.dict()
        )

    def remove_included_users(self, watchlist_id: str, user_ids: Union[str, List[str]]):
        """
        Remove included users from a watchlist.

        **Parameters**:

        * **watchlist**: `str` (required) - Watchlist ID.
        * **user_ids**: `str`, `List[str]` (required) - List of unique user IDs to remove from the watchlist. A maximum
            of 100 users can be processed in a single request.

        **Returns**: A `requests.Response` indicating success.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )

        data = UpdateIncludedUsersRequest(
            userIds=user_ids if isinstance(user_ids, List) else [user_ids],
            watchlistId=watchlist_id,
        )
        return self._parent.session.post(
            url=f"{self._uri}/{watchlist_id}/included-users/delete", json=data.dict()
        )

    def get_included_user(self, watchlist_id: str, user_id: str) -> WatchlistUser:
        """
        Get an included user from a watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.
        * **user_id**: `str` (required) - Unique user ID.

        **Returns**: A [`WatchlistUser`][watchlistuser-model] object.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        response = self._parent.session.get(
            url=f"{self._uri}/{watchlist_id}/included-users/{user_id}"
        )
        return WatchlistUser.parse_response(response)

    def list_included_users(self, watchlist_id: str) -> IncludedUsersList:
        """
        List individual users included on a watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.

        **Returns**: An [`IncludedUsersList`][includeduserslist-model] object.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )

        response = self._parent.session.get(
            url=f"{self._uri}/{watchlist_id}/included-users"
        )
        return IncludedUsersList.parse_response(response)

    def add_excluded_users(self, watchlist_id: str, user_ids: Union[str, List[str]]):
        """
        Exclude individual users from a watchlist.

        **Parameters**:

        * **watchlist**: `str` (required) - Watchlist ID.
        * **user_ids**: `str`, `List[str]` (required) - List of unique user IDs to exclude from the watchlist. A maximum
            of 100 users can be processed in a single request.

        **Returns**: A `requests.Response` indicating success.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        data = UpdateExcludedUsersRequest(
            userIds=user_ids if isinstance(user_ids, List) else [user_ids],
        )
        return self._parent.session.post(
            url=f"{self._uri}/{watchlist_id}/excluded-users/add", json=data.dict()
        )

    def remove_excluded_users(self, watchlist_id: str, user_ids: Union[str, List[str]]):
        """
        Remove excluded users from a watchlist.

        **Parameters**:

        * **watchlist**: `str` (required) - Watchlist ID.
        * **user_ids**: `str`, `List[str]` (required) - List of unique user IDs to remove from the exclusion list. A
            maximum of 100 users can be processed in a single request.

        **Returns**: A `requests.Response` indicating success.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        data = UpdateExcludedUsersRequest(
            userIds=user_ids if isinstance(user_ids, List) else [user_ids],
        )
        return self._parent.session.post(
            url=f"{self._uri}/{watchlist_id}/excluded-users/delete", json=data.dict()
        )

    def list_excluded_users(self, watchlist_id: str) -> ExcludedUsersList:
        """
        List individual users excluded from a watchlist.

        * **watchlist_id**: `str` (required) - Watchlist ID.

        **Returns**: An [`ExcludedUsersList`][excludeduserslist-model] object.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        response = self._parent.session.get(
            f"{self._uri}/{watchlist_id}/excluded-users"
        )
        return ExcludedUsersList.parse_response(response)

    def get_excluded_user(self, watchlist_id: str, user_id: str) -> WatchlistUser:
        """
        Get an excluded user from a watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.
        * **user_id**: `str` (required) - Unique user ID.

        **Returns**: A [`WatchlistUser`][watchlistuser-model] object.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        response = self._parent.session.get(
            f"{self._uri}/{watchlist_id}/excluded-users/{user_id}"
        )
        return WatchlistUser.parse_response(response)

    def add_directory_groups(self, watchlist_id: str, group_ids: Union[str, List[str]]):
        """
        Include directory groups on a watchlist. Use the `directory_groups` client to see available directory groups.

        **Parameters**:

        * **watchlist**: `str` (required) - Watchlist ID.
        * **group_ids**: `str`, `List[str]` (required) - List of directory group IDs to include on the watchlist.

        **Returns**: A `requests.Response` indicating success.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        data = UpdateIncludedDirectoryGroupsRequest(
            groupIds=group_ids if isinstance(group_ids, List) else [group_ids]
        )
        return self._parent.session.post(
            url=f"{self._uri}/{watchlist_id}/included-directory-groups/add",
            json=data.dict(),
        )

    def remove_directory_groups(
        self, watchlist_id: str, group_ids: Union[str, List[str]]
    ):
        """
        Remove included directory groups from a watchlist.

        **Parameters**:

        * **watchlist**: `str` (required) - Watchlist ID.
        * **group_ids**: `str`, `List[str]` (required) - List of directory group IDs to remove from the watchlist.

        **Returns**: A `requests.Response` indicating success.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        data = UpdateIncludedDirectoryGroupsRequest(
            groupIds=group_ids if isinstance(group_ids, List) else [group_ids]
        )
        return self._parent.session.post(
            url=f"{self._uri}/{watchlist_id}/included-directory-groups/delete",
            json=data.dict(),
        )

    def list_directory_groups(self, watchlist_id: str) -> IncludedDirectoryGroupsList:
        """
        List directory groups included on a watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.

        **Returns**: An [`IncludedUsersList`][includeduserslist-model] object.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )

        response = self._parent.session.get(
            f"{self._uri}/{watchlist_id}/included-directory-groups"
        )
        return IncludedDirectoryGroupsList.parse_response(response)

    def get_directory_group(
        self, watchlist_id: str, group_id: str
    ) -> IncludedDirectoryGroup:
        """
        Get an included directory group from a watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.
        * **group_id**: `str` (required) - Directory group ID.

        **Returns**: An [`IncludedDirectoryGroup`][includeddirectorygroup-model] object.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )

        response = self._parent.session.get(
            f"{self._uri}/{watchlist_id}/included-directory-groups/{group_id}"
        )
        return IncludedDirectoryGroup.parse_response(response)

    def add_departments(
        self,
        watchlist_id: str,
        departments: Union[str, List[str]],
    ):
        """
        Include departments on a watchlist. Use the `departments` client to see available departments.

        **Parameters**:

        * **watchlist**: `str` (required) - Watchlist ID.
        * **departments**: `str`, `List[str]` (required) - List of departments to include on the watchlist.

        **Returns**: A `requests.Response` indicating success.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        data = UpdateIncludedDepartmentsRequest(
            departments=departments if isinstance(departments, List) else [departments]
        )
        return self._parent.session.post(
            url=f"{self._uri}/{watchlist_id}/included-departments/add", json=data.dict()
        )

    def remove_departments(
        self,
        watchlist_id: str,
        departments: Union[str, List[str]],
    ):
        """
        Remove included departments from a watchlist.

        **Parameters**:

        * **watchlist**: `str` - Watchlist ID or a watchlist type.  An ID must be provided for custom watchlists.
        * **departments**: `str`, `List[str]` (required) - List of departments to remove from the watchlist.

        **Returns**: A `requests.Response` indicating success.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        data = UpdateIncludedDepartmentsRequest(
            departments=departments if isinstance(departments, List) else [departments]
        )
        return self._parent.session.post(
            url=f"{self._uri}/{watchlist_id}/included-departments/delete",
            json=data.dict(),
        )

    def list_departments(self, watchlist_id: str) -> IncludedDepartmentsList:
        """
        List departments included on a watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.

        **Returns**: An [`IncludedDepartmentsList`][includeddepartmentslist-model] object.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        response = self._parent.session.get(
            f"{self._uri}/{watchlist_id}/included-departments"
        )
        return IncludedDepartmentsList.parse_response(response)

    def get_department(self, watchlist_id: str, department: str) -> IncludedDepartment:
        """
        Get an included department from a watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.
        * **department**: `str` (required) - A included department.

        **Returns**: An [`IncludedDepartment`][includeddepartment-model] object.
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )
        response = self._parent.session.get(
            f"{self._uri}/{watchlist_id}/included-departments/{department}"
        )
        return IncludedDepartment.parse_response(response)

    def get_id_by_name(self, name: Union[str, WatchlistType]):
        """
        Get a watchlist ID by either its type (ex: `DEPARTING_EMPLOYEE`) or its title in the case of `CUSTOM` watchlists.

        **Parameters**:

        * **name**: `str`, [`WatchlistType`][watchlist-types] (required) - A `WatchlistType` or in the case of `CUSTOM` watchlists, the watchlist `title`.

        **Returns**: A watchlist ID (`str`).
        """
        warn(
            "WatchlistsV1 is deprecated and replaced by WatchlistsV2.",
            DeprecationWarning,
            stacklevel=2,
        )

        def _lookup_ids(self):
            """Map watchlist types to IDs, if they exist."""
            self._watchlist_type_id_map = {}
            watchlists = self.get_page(page_size=100).watchlists
            for item in watchlists:
                if item.list_type == "CUSTOM":
                    # store title for custom lists instead of list_type
                    self._watchlist_type_id_map[item.title] = item.watchlist_id
                self._watchlist_type_id_map[item.list_type] = item.watchlist_id

        watchlist_id = self._watchlist_type_id_map.get(name)
        if not watchlist_id:
            # if not found, reset ID cache
            _lookup_ids(self)
            watchlist_id = self._watchlist_type_id_map.get(name)
            if not watchlist_id:
                raise WatchlistNotFoundError(name)
        return watchlist_id



import click

from _incydr_cli.cmds.utils import actor_lookup
from _incydr_cli.cmds.utils import user_lookup
from _incydr_sdk.core.client import Client

checkpoint_option = click.option(
    "--checkpoint",
    "checkpoint_name",
    default=None,
    help="Use a checkpoint with the given name to only get search results that were not previously retrieved. "
    "If a checkpoint for the search with the given name doesn't exist, it will be created on the first run. "
    "Subsequent CLI runs with this option and the same name will use the stored checkpoint to modify the search query and then update the stored checkpoint."
    "Checkpointing is most accurate with json outputs.  For table and csv formats, checkpointing will track the last returned event in the table.",
)


def user_lookup_callback(ctx, param, value):
    if not value:
        return
    # only call user_lookup if username to prevent unnecessary client inits with obj()
    if "@" in str(value):
        return user_lookup(Client(), value)
    return value


def actor_lookup_callback(ctx, param, value):
    if not value:
        return
    # only call user_lookup if username to prevent unnecessary client inits with obj()
    if "@" in str(value):
        return actor_lookup(Client(), value)
    return value



import itertools
from datetime import datetime
from typing import List
from typing import Optional
from typing import Union

from _incydr_sdk.enums import SortDirection
from _incydr_sdk.enums.sessions import ContentInspectionStatuses
from _incydr_sdk.enums.sessions import SessionStates
from _incydr_sdk.enums.sessions import SortKeys
from _incydr_sdk.queries.utils import parse_ts_to_ms_ts
from _incydr_sdk.sessions.models.models import SessionsChangeStateRequest
from _incydr_sdk.sessions.models.models import SessionsCriteriaRequest
from _incydr_sdk.sessions.models.models import SessionsQueryRequest
from _incydr_sdk.sessions.models.response import Session
from _incydr_sdk.sessions.models.response import SessionEvents
from _incydr_sdk.sessions.models.response import SessionsPage


class SessionsV1:
    """
    Client for `/v1/sessions` endpoints.

    Usage example:

        >>> import incydr
        >>> from incydr.enums.items import SessionStates
        >>> client = incydr.Client(**kwargs)
        >>> client.items.v1.change_state("<session_id>", SessionStates.CLOSED)
    """

    def __init__(self, parent):
        self._parent = parent

    def get_page(
        self,
        actor_id: str = None,
        start_time: Union[str, datetime, int] = None,
        end_time: Union[str, datetime, int] = None,
        has_alerts: bool = True,
        sort_key: Optional[SortKeys] = None,
        risk_indicators: List[str] = None,
        sort_dir: Optional[SortDirection] = None,
        states: List[SessionStates] = None,
        severities: Union[List[int], int] = None,
        rule_ids: Union[List[str], str] = None,
        watchlist_ids: Union[List[str], str] = None,
        page_num: int = 0,
        page_size: int = 50,
        content_inspection_status: Optional[ContentInspectionStatuses] = None,
    ):
        """
        Get a page of items.

        Filter results by passing the appropriate parameters.

        **Parameters**:

        * **actor_id**: `str | None` - Only include items generated by this actor.
        * **start_time**: `datetime | str | int | None` - Only include items beginning on or after this date and time.  Can be a date-formatted string, a `datetime` instance, or a POSIX `int` timestamp.
        * **end_time**: `datetime | str | int | None` - Only include items beginning before this date and time.  Can be a date-formatted string, a `datetime` instance, or a POSIX `int` timestamp.
        * **has_alerts**: `bool` - Only include items that have a matching alert status. Defaults to `True`.
        * **sort_key**: [`SortKeys`][items-sort-keys] - `end_time` or `score`. Value on which the results will be sorted. Defaults to `end time`.
        * **risk_indicators**: `List[str] | None` - List of risk indicator IDs that must be present on the items before they are returned.
        * **sort_dir**: `SortDirection` - `asc` for ascending or `desc` for descending. The direction in which to sort the response based on the corresponding key. Defaults to `desc`.
        * **states**: List[[`SessionStates`][items-session-states]] - Optional list of one or more session states to filter upon. Only include items that include these matching states.
        * **severities**: `List[int] | None` - Only include items that have the matching severity value(s). 0 = no risk, 1 = low, 2 = moderate, 3 = high, 4 = critical
        * **rule_ids**: `List[str] | None` - Optional list of one or more rule ids to filter upon.
        * **watchlist_ids**: `List[str] | None` - Optional list of one or more watchlist ids to filter upon.
        * **page_num**: `int` - Page number for results, starting at 0.
        * **page_size**: `int` - Max number of results to return per page, between 1 and 50 inclusive. Defaults to 50.
        * **content_inspection_status**: `List[[ContentInspectionStatuses][items-content-inspection-statuses]] | None` - The content inspection status(es) to limit the search to.

        **Returns**: A [`SessionsPage`][sessionspage-model] object.
        """

        # Parse timestamps
        if start_time and not isinstance(start_time, (int, float)):
            start_time = parse_ts_to_ms_ts(start_time)
        if end_time and not isinstance(end_time, (int, float)):
            end_time = parse_ts_to_ms_ts(end_time)

        if states and not isinstance(states, List):
            states = [states]
        if rule_ids and not isinstance(rule_ids, List):
            rule_ids = [rule_ids]
        if watchlist_ids and not isinstance(watchlist_ids, List):
            watchlist_ids = [watchlist_ids]
        if severities and not isinstance(severities, List):
            severities = [severities]

        data = SessionsQueryRequest(
            actor_id=actor_id,
            on_or_after=start_time,
            before=end_time,
            has_alerts=str(has_alerts).lower() if has_alerts is not None else None,
            order_by=sort_key,
            risk_indicators=risk_indicators,
            sort_direction=sort_dir,
            state=states,
            severity=severities,
            rule_id=rule_ids,
            watchlist_id=watchlist_ids,
            page_number=page_num,
            page_size=page_size,
            content_inspection_status=content_inspection_status,
        )
        response = self._parent.session.get("/v1/sessions", params=data.dict())
        return SessionsPage.parse_response(response)

    def iter_all(
        self,
        actor_id: str = None,
        start_time: Union[str, datetime, int] = None,
        end_time: Union[str, datetime, int] = None,
        has_alerts: bool = True,
        sort_key: Optional[SortKeys] = None,
        risk_indicators: List[str] = None,
        sort_dir: Optional[SortDirection] = None,
        states: List[SessionStates] = None,
        severities: List[int] = None,
        rule_ids: List[str] = None,
        watchlist_ids: List[str] = None,
        page_size: int = 50,
        content_inspection_status: Optional[ContentInspectionStatuses] = None,
    ):
        """
        Iterate over all items.

        Accepts the same parameters as `.get_page()` excepting `page_num`.

        **Returns**: A generator yielding individual [`Session`][session-model] objects.
        """
        for page_num in itertools.count(0):
            page = self.get_page(
                actor_id=actor_id,
                start_time=start_time,
                end_time=end_time,
                has_alerts=has_alerts,
                sort_key=sort_key,
                risk_indicators=risk_indicators,
                sort_dir=sort_dir,
                states=states,
                severities=severities,
                rule_ids=rule_ids,
                watchlist_ids=watchlist_ids,
                page_num=page_num,
                page_size=page_size,
                content_inspection_status=content_inspection_status,
            )
            yield from page.items
            if len(page.items) < page_size:
                break

    def get_session_details(self, session_id: str):
        """
        Get details of a session.

        **Parameters**:

        * **session_id**: `str` (required) - The session ID.

        **Returns**: A [`Session`][session-model] object representing the session.
        """
        response = self._parent.session.get(f"/v1/sessions/{session_id}")
        return Session.parse_response(response)

    def get_session_events(self, session_id: str):
        """
        Gets details for the events associated with alerted-on session activity.

        Returns the same response object as the file event client search method.

        **Parameters**:

        * **session_id**: `str` (required) - The session ID.

        **Returns**: A [`FileEventsPage`][fileeventspage-model] object.
        """
        response = self._parent.session.get(f"/v1/sessions/{session_id}/events")
        return SessionEvents.parse_response(response).query_result

    def update_state_by_id(
        self, session_ids: Union[str, List[str]], new_state: SessionStates
    ):
        """
        Change the state of a one or more items specified by ID.

        Processes up to 100 session IDs at a time and continues to make subsequent API calls until all indicated items are updated.

        **Parameters**:

        * **session_id**: `str | List[str]` (required) - One or more session IDs.
        * **state**: [`SessionStates`][items-session-states] - The new state for the desired items.

        **Returns**: An array of all `requests.Response` objects received during processing.
        """
        if isinstance(session_ids, str):
            session_ids = [session_ids]
        results = []
        ids = iter(session_ids)
        while True:
            chunk = list(itertools.islice(ids, 100))
            if not chunk:
                break
            else:
                data = SessionsChangeStateRequest(ids=session_ids, newState=new_state)
                response = self._parent.session.post(
                    "/v1/sessions/change-state", json=data.dict()
                )
                results.append(response)
        return results

    def update_state_by_criteria(
        self,
        new_state: SessionStates,
        actor_id: str = None,
        start_time: Union[str, datetime, int] = None,
        end_time: Union[str, datetime, int] = None,
        has_alerts: bool = True,
        risk_indicators: List[str] = None,
        states: List[SessionStates] = None,
        severities: Union[List[int], int] = None,
        rule_ids: Union[List[str], str] = None,
        watchlist_ids: Union[List[str], str] = None,
        content_inspection_status: Optional[ContentInspectionStatuses] = None,
    ):
        """
        Change the state of all items matching the filter criteria.

        Makes an initial API call to update the desired items to the `state`.
        Processes up to 500 session IDs at a time and continues to make subsequent API calls until all items matching the criteria are updated.

        **Parameters**:

        * **actor_id**: `str | None` - The ID of the actor to limit the search to.
        * **start_time**: `datetime | str | int | None` - Only include items beginning on or after this date and time.  Can be a date-formatted string, a `datetime` instance, or a POSIX `int` timestamp.
        * **end_time**: `datetime | str | int | None` - Only include items beginning before this date and time.  Can be a date-formatted string, a `datetime` instance, or a POSIX `int` timestamp.
        * **has_alerts**: `bool` - Only include items that have a matching alert status. Defaults to `True`.
        * **sort_key**: [`SortKeys`][items-sort-keys] - `end_time` or `score`. Value on which the results will be sorted. Defaults to `end time`.
        * **risk_indicators**: `List[str] | None` - List of risk indicator IDs that must be present on the items before they are returned.
        * **sort_dir**: `SortDirection` - `asc` for ascending or `desc` for descending. The direction in which to sort the response based on the corresponding key. Defaults to `desc`.
        * **states**: List[[`SessionStates`][items-session-states]] - Only include items that have a matching state.
        * **severities**: `List[int | None` - Only include items that have a matching severity value. 0 = no risk, 1 = low, 2 = moderate, 3 = high, 4 = critical
        * **rule_ids**: `List[str] | None` - Optional list of rule ids to filter upon.
        * **watchlist_ids**: `List[str] | None` - Optional list of watchlist ids to filter upon.
        * **page_num**: `int` - Page number for results, starting at 1.
        * **page_size**: `int` - Max number of results to return per page, between 1 and 50 inclusive. Defaults to 50.
        * **content_inspection_status**: `List[[ContentInspectionStatuses][items-content-inspection-statuses]] | None` - The content inspection status(es) to limit the search to.

        **Returns**: An array of all `requests.Response` objects received during processing.
        """

        # Parse timestamps
        if start_time and not isinstance(start_time, (int, float)):
            start_time = parse_ts_to_ms_ts(start_time)
        if end_time and not isinstance(end_time, (int, float)):
            end_time = parse_ts_to_ms_ts(end_time)

        if states and not isinstance(states, List):
            states = [states]
        if rule_ids and not isinstance(rule_ids, List):
            rule_ids = [rule_ids]
        if watchlist_ids and not isinstance(watchlist_ids, List):
            watchlist_ids = [watchlist_ids]
        if severities and not isinstance(severities, List):
            severities = [severities]

        data = SessionsCriteriaRequest(
            actor_id=actor_id,
            on_or_after=start_time,
            before=end_time,
            has_alerts=str(has_alerts).lower() if has_alerts is not None else None,
            risk_indicators=risk_indicators,
            state=states,
            severity=severities,
            rule_id=rule_ids,
            watchlist_id=watchlist_ids,
            content_inspection_status=content_inspection_status,
        )
        continuation_token = None
        initial_request_made = False
        results = []

        # make initial request & continue processing continuation tokens
        while continuation_token is not None or not initial_request_made:
            response = self._parent.session.post(
                "/v1/sessions/change-states",
                params=data.dict(),
                json={"continuationToken": continuation_token, "newState": new_state},
            )
            results.append(response)
            initial_request_made = True
            try:
                continuation_token = response.json()["continuationToken"]
            except IndexError:
                continuation_token = None
        return results

    def add_note(self, session_id: str, note_content: str):
        """
        Add a note to a session, specified by ID.

        **Parameters**:

        * **session_id**: `str` (required) - The session ID.
        * **note_content**: `str` (required) -  The note content to add. Max 2000 characters.

        **Returns**: A `requests.Response` object indicating success.
        """
        if len(note_content) > 2000:
            raise ValueError("note_content has a 2000 character max limit.")

        return self._parent.session.post(
            f"/v1/sessions/{session_id}/add-note", json={"noteContent": note_content}
        )


class SessionsClient:
    def __init__(self, parent):
        self._parent = parent
        self._v1 = None

    @property
    def v1(self):
        if self._v1 is None:
            self._v1 = SessionsV1(self._parent)
        return self._v1



from pathlib import Path


class FilesClient:
    def __init__(self, parent):
        self._parent = parent
        self._v1 = None

    @property
    def v1(self):
        if self._v1 is None:
            self._v1 = FilesV1(self._parent)
        return self._v1


class FilesV1:
    """Client for `/v1/files` endpoints.

    Usage example:

        >>> import incydr
        >>>
        >>> client = incydr.Client(**kwargs)
        >>> client.files.v1.download_file_by_sha256("example_hash", "./testfile.test")
    """

    def __init__(self, parent):
        self._parent = parent

    def download_file_by_sha256(self, sha256: str, target_path: Path) -> Path:
        """Download a file that matches the given SHA256 hash.

        **Parameters:**

        * **sh256**: `str` (required) The SHA256 hash matching the file you wish to download.
        * **target_path**: `Path | str` a string or `pathlib.Path` object that represents the target file path and
            name to which the file will be saved to.

        **Returns**: A `pathlib.Path` object representing the location of the downloaded file.
        """
        target = Path(
            target_path
        )  # ensure that target is a path even if we're given a string
        response = self._parent.session.get(f"/v1/files/get-file-by-sha256/{sha256}")
        target.write_bytes(response.content)
        return target

    def stream_file_by_sha256(self, sha256: str):
        """Stream a file that matches the given SHA256 hash.

        **Example usage:**
        ```
        >>> with sdk.files.v1.stream_file_by_sha256("example_hash") as response:
        >>>     with open("./testfile.zip", "wb") as file:
        >>>         for chunk in response.iter_content(chunk_size=128):
        >>>             file.write(chunk)
        ```

        **Parameters:**

        * **sh256**: `str` (required) The SHA256 hash matching the file you wish to download.

        **Returns**: A `requests.Response` object with a stream of the requested file.
        """
        return self._parent.session.get(
            f"/v1/files/get-file-by-sha256/{sha256}", stream=True
        )



from typing import List
from typing import Optional

from pydantic import BaseModel
from pydantic import Field

from _incydr_sdk.core.models import Model
from _incydr_sdk.enums import SortDirection
from _incydr_sdk.enums.sessions import ContentInspectionStatuses
from _incydr_sdk.enums.sessions import SessionStates
from _incydr_sdk.enums.sessions import SortKeys


class ContentInspectionEvent(Model):
    event_id: Optional[str] = Field(alias="eventId")
    pii_type: Optional[List[str]] = Field(alias="piiType")
    status: Optional[str]


class ContentInspectionResult(Model):
    detected_on_alerts: List[str] = Field(
        alias="detectedOnAlerts",
        description="A list of content categories or types found on events which triggered alerts.",
    )


class Note(Model):
    content: Optional[str]
    id: Optional[str]
    source_timestamp: Optional[int] = Field(alias="sourceTimestamp")
    user_id: Optional[str] = Field(alias="userId")


class RiskIndicator(Model):
    event_count: Optional[int] = Field(alias="eventCount")
    id: Optional[str]
    name: Optional[str]
    weight: Optional[int]


class Score(Model):
    score: Optional[int]
    severity: Optional[int]
    source_timestamp: Optional[int] = Field(alias="sourceTimestamp")


class State(Model):
    source_timestamp: Optional[int] = Field(alias="sourceTimestamp")
    state: SessionStates
    user_id: Optional[str] = Field(
        alias="userId", description="A User ID. (Deprecated)"
    )


class Alert(Model):
    """ """

    alert_id: Optional[str] = Field(alias="alertId")
    lesson_id: Optional[str] = Field(alias="lessonId")
    rule_id: Optional[str] = Field(alias="ruleId")


class SessionsCriteriaRequest(BaseModel):
    actor_id: Optional[str]
    on_or_after: Optional[int]
    before: Optional[int]
    has_alerts: Optional[str]
    risk_indicators: Optional[List[str]]
    state: Optional[List[SessionStates]]
    severity: Optional[List[int]]
    rule_id: Optional[List[str]]
    watchlist_id: Optional[List[str]]
    content_inspection_status: Optional[ContentInspectionStatuses]

    class Config:
        use_enum_values = True


class SessionsQueryRequest(SessionsCriteriaRequest):
    order_by: Optional[SortKeys]
    sort_direction: Optional[SortDirection]
    page_number: Optional[int]
    page_size: Optional[int]


class SessionsChangeStateRequest(BaseModel):
    ids: Optional[List[str]]
    newState: Optional[str]



import difflib
import re

import click
from requests import HTTPError

from _incydr_cli.exceptions import IncydrCLIException
from _incydr_cli.exceptions import LoggedCLIError
from _incydr_sdk.core.settings import IncydrSettings
from _incydr_sdk.exceptions import AuthMissingError
from _incydr_sdk.exceptions import IncydrException

_DIFFLIB_CUT_OFF = 0.6


try:
    import rich

    from _incydr_cli import rich_utils

except ImportError:  # pragma: nocover
    rich = None  # type: ignore


class IncydrCommand(click.Command):
    def __init__(self, *args, **kwargs):
        # self.rich_markup_mode should be one of ["markdown", "rich", None]
        self.rich_markup_mode = "rich"
        super().__init__(*args, **kwargs)

    def format_help(self, ctx: click.Context, formatter: click.HelpFormatter) -> None:
        if not rich:
            return super().format_help(ctx, formatter)
        return rich_utils.rich_format_help(
            obj=self,
            ctx=ctx,
            markup_mode=self.rich_markup_mode,
        )


class IncydrGroup(click.Group):
    def __init__(self, *args, **kwargs):
        self.rich_markup_mode = "rich"
        super().__init__(*args, **kwargs)

    def format_help(self, ctx: click.Context, formatter: click.HelpFormatter) -> None:
        if not rich:
            return super().format_help(ctx, formatter)
        return rich_utils.rich_format_help(
            obj=self,
            ctx=ctx,
            markup_mode=self.rich_markup_mode,
        )


class ExceptionHandlingGroup(IncydrGroup):
    """A `click.Group` subclass to add custom exception handling."""

    _original_args = None

    def make_context(self, info_name, args, parent=None, **extra):
        # grab the original command line arguments for logging purposes
        self._original_args = " ".join(args)

        return super().make_context(info_name, args, parent=parent, **extra)

    def invoke(self, ctx):
        settings = IncydrSettings(url="", api_client_id="", api_client_secret="")
        try:
            return super().invoke(ctx)
        except click.UsageError as err:
            self._suggest_cmd(err)
        except LoggedCLIError:
            raise
        except click.exceptions.Exit:
            raise
        except AuthMissingError as err:
            missing_vars = "\n".join(
                f"  - INCYDR_{key.upper()}" for key in err.error_keys
            )
            msg = (
                f"Missing authentication variables in environment.\n\n{missing_vars}"
                "\n\nSee https://developer.code42.com/cli/getting_started/#authentication"
            )
            settings.logger.error(msg)
            raise IncydrCLIException(msg)

        except IncydrException as err:
            # log error and raise custom error to print error message to console
            settings._log_error(err, self._original_args)
            raise IncydrCLIException(err.args[0])
        except click.ClickException:
            raise
        except HTTPError as err:
            # log error with traceback and print error code with brief error message to console
            settings._log_verbose_error(self._original_args, err.request)
            raise LoggedCLIError(err.args[0])
        except OSError:
            raise
        except Exception:
            # log error with traceback and print message pointing user to logs
            settings._log_verbose_error(self._original_args)
            raise LoggedCLIError("Unknown problem occurred.")

    @staticmethod
    def _suggest_cmd(usage_err):
        """Handles fuzzy suggestion of commands that are close to the bad command entered."""
        if usage_err.message is not None:
            match = re.match("No such command '(.*)'.", usage_err.message)
            if match:
                bad_arg = match.groups()[0]
                available_commands = list(usage_err.ctx.command.commands.keys())
                suggested_commands = difflib.get_close_matches(
                    bad_arg, available_commands, cutoff=_DIFFLIB_CUT_OFF
                )
                if not suggested_commands:
                    raise usage_err
                usage_err.message = (
                    f"No such command '{bad_arg}'. "
                    f"Did you mean {' or '.join(suggested_commands)}?"
                )
        raise usage_err


def incompatible_with(incompatible_opts):
    """Factory for creating custom `click.Option` subclasses that enforce incompatibility with the
    option strings passed to this function.
    """

    if isinstance(incompatible_opts, str):
        incompatible_opts = [incompatible_opts]

    class IncompatibleOption(click.Option):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)

        def handle_parse_result(self, ctx, opts, args):
            # if None it means we're in autocomplete mode and don't want to validate
            if ctx.obj is not None:
                found_incompatible = ", ".join(
                    [
                        f"--{opt.replace('_', '-')}"
                        for opt in opts
                        if opt in incompatible_opts
                    ]
                )
                if self.name in opts and found_incompatible:
                    name = self.name.replace("_", "-")
                    raise click.BadOptionUsage(
                        option_name=self.name,
                        message=f"--{name} can't be used with: {found_incompatible}",
                    )
            return super().handle_parse_result(ctx, opts, args)

    return IncompatibleOption



import os

import click
from click._compat import get_text_stderr

from _incydr_cli import ERROR_LOG_FILE_NAME
from _incydr_cli import get_user_project_path


class IncydrCLIException(click.ClickException):
    """Wrapper class for Incydr CLI exceptions. The `message` param automatically gets logged to error file and printed
    to stderr in red text. If `help` param is provided, it will also be printed to stderr after the
    message but not logged to file.
    """

    def __init__(self, message, help=None):
        self.help = help
        super().__init__(message)

    def show(self, file=None):
        """Override default `show` to print CLI errors in red text."""
        if file is None:
            file = get_text_stderr()
        click.secho(f"ERROR: {self.format_message()}", file=file, fg="red")
        if self.help:
            click.echo(self.help, err=True)


class LoggedCLIError(IncydrCLIException):
    """Exception to be raised when wanting to point users to error logs for error details.

    If `message` param is provided it will be printed to screen along with message on where to
    find error details in the log.
    """

    def __init__(self, message=None):
        self.message = message
        super().__init__(message)

    def format_message(self):
        path = os.path.join(get_user_project_path("log"), ERROR_LOG_FILE_NAME)
        locations_message = f"View details in {path}"
        return (
            f"{self.message}\n{locations_message}"
            if self.message
            else locations_message
        )



from itertools import count
from typing import Iterator
from typing import List

from requests import Response

from _incydr_sdk.enums import SortDirection
from _incydr_sdk.enums.trusted_activities import ActivityType
from _incydr_sdk.enums.trusted_activities import CloudShareApps
from _incydr_sdk.enums.trusted_activities import CloudSyncApps
from _incydr_sdk.enums.trusted_activities import EmailServices
from _incydr_sdk.enums.trusted_activities import Name
from _incydr_sdk.enums.trusted_activities import SortKeys
from _incydr_sdk.exceptions import IncydrException
from _incydr_sdk.trusted_activities.models import ActivityAction
from _incydr_sdk.trusted_activities.models import ActivityActionGroup
from _incydr_sdk.trusted_activities.models import CreateTrustedActivityRequest
from _incydr_sdk.trusted_activities.models import ProviderObject
from _incydr_sdk.trusted_activities.models import QueryTrustedActivitiesRequest
from _incydr_sdk.trusted_activities.models import TrustedActivitiesPage
from _incydr_sdk.trusted_activities.models import TrustedActivity
from _incydr_sdk.trusted_activities.models import UpdateTrustedActivity


class TrustedActivitiesV2:
    """
    Client for `/v2/trusted-activites` endpoints.

    Usage example:

        >>> import incydr
        >>> client = incydr.Client(**kwargs)
        >>> client.trusted_activities.v2.get_page()

    """

    def __init__(self, parent):
        self._parent = parent

    def get_trusted_activity(self, activity_id: str) -> TrustedActivity:
        """
        Get a single trusted activity.

        **Parameters:**

        * **activity_id**: `str` (required) - The unique ID for the trusted activity.

        **Returns**: A [`TrustedActivity`][trustedactivity-model] object representing the trusted activity.
        """

        response = self._parent.session.get(f"/v2/trusted-activities/{activity_id}")
        return TrustedActivity.parse_response(response)

    def get_page(
        self,
        page_num: int = 1,
        page_size: int = None,
        activity_type: ActivityType = None,
        sort_key: SortKeys = None,
        sort_direction: SortDirection = None,
    ) -> TrustedActivitiesPage:
        """
        Get a page of trusted activities.
        Filter results by passing the appropriate parameters:

        **Parameters**:

        * **page_num**: `int` - Page number for results, starting at 1.
        * **page_size**: `int` - Max number of results to return per page. Defaults to client's `page_size` setting.
        * **activity_type**: `ActivityType` - The type of the trusted activity.
        * **sort_key**: `SortKeys` - The key by which to sort the returned list.
        * **sort_dir**: `SortDirection` - The order in which to sort the returned list.

        **Returns**: A [`TrustedActivitiesPage`][trustedactivitiespage-model] object.
        """

        page_size = page_size or self._parent.settings.page_size
        data = QueryTrustedActivitiesRequest(
            page_num=page_num,
            page_size=page_size,
            activity_type=activity_type,
            sort_key=sort_key,
            sort_direction=sort_direction,
        )
        response = self._parent.session.get(
            "/v2/trusted-activities", params=data.dict()
        )
        return TrustedActivitiesPage.parse_response(response)

    def iter_all(
        self,
        page_size: int = None,
        activity_type: ActivityType = None,
        sort_key: SortKeys = None,
        sort_direction: SortDirection = None,
    ) -> Iterator[TrustedActivity]:
        """
        Iterate over all trusted activities.

        Accepts the same parameters as `.get_page()` except `page_num`.

        **Returns**: A generator yielding individual [`TrustedActivity`][trustedactivity-model] objects.
        """

        page_size = page_size or self._parent.settings.page_size
        for page_num in count(1):
            page = self.get_page(
                page_num=page_num,
                page_size=page_size,
                activity_type=activity_type,
                sort_key=sort_key,
                sort_direction=sort_direction,
            )
            yield from page.trusted_activities
            if len(page.trusted_activities) < page_size:
                break

    def add_domain(
        self,
        domain: str,
        description: str = None,
        file_upload: bool = None,
        cloud_sync_services: List[CloudSyncApps] = None,
        cloud_share_services: List[CloudShareApps] = None,
        email_share_services: List[EmailServices] = None,
        git_push: bool = None,
    ) -> TrustedActivity:
        """
        Trust activity across an entire domain.

        **Parameters:**

        * **domain**: `str` (required) - Domain to trust.
        * **description**: `str` - Optional description of the trusted activity.
        * **file_upload**: `bool` - Whether to trust activity if the tab URL or tab title includes this domain.
        * **cloud_sync_services**: `List[CloudSyncApps]` - Activity is trusted if the username signed in to the
            sync app is on this domain.  Supported cloud storage apps for file syncing are
            `BOX`, `GOOGLE_DRIVE`, `ICLOUD` and/or `ONE_DRIVE`.
        * **cloud_share_services**: `List[CloudShareApps]` - Activity is trusted if the user it's shared with is on this domain.
            Supported cloud storage services for file sharing are `BOX`, `GOOGLE_DRIVE` and/or `ONE_DRIVE`. You must
            have a cloud connector configured for your tenant to support this trusted action.
        * **email_share_services**: `List[EmailServices]` - Activity is trusted if the email recipient is on this domain.
            Supported email services are `GMAIL` and/or `MICROSOFT_365`.  You must have an email connector configured
            for your tenant to support this trusted action.
        * **git_push**: `bool` - Whether to trust Git push events to this domain.

        **Returns**: A [`TrustedActivity`][trustedactivity-model] object representing
        the newly created trusted activity.
        """

        activity_actions = []

        # FILE UPLOAD
        if file_upload:
            activity_actions.append(ActivityAction(type=ActivityType.FILE_UPLOAD))

        # GIT PUSH
        if git_push:
            activity_actions.append(ActivityAction(type=ActivityType.GIT_PUSH))

        # CLOUD SYNC SERVICES
        services = [
            (
                cloud_sync_services,
                CloudSyncApps,
                ActivityType.CLOUD_SYNC,
            ),  # CLOUD SYNC SERVICES
            (
                cloud_share_services,
                CloudShareApps,
                ActivityType.CLOUD_SHARE,
            ),  # CLOUD_SHARE_SERVICES
            (
                email_share_services,
                EmailServices,
                ActivityType.EMAIL,
            ),  # # EMAIL_SHARE_SERVICES
        ]
        for element in services:
            service, enum, activity_type = element
            if not service:
                continue
            providers = []
            for provider in service:
                providers.append(ProviderObject(name=enum(provider)))
            activity_actions.append(
                ActivityAction(providers=providers, type=activity_type)
            )

        if len(activity_actions) < 1:
            raise MissingActivityActionGroupsError()

        data = CreateTrustedActivityRequest(
            type=ActivityType.DOMAIN,
            value=domain,
            description=description,
            activityActionGroups=[
                ActivityActionGroup(activityActions=activity_actions, name=Name.DEFAULT)
            ],
        )

        response = self._parent.session.post(
            url="/v2/trusted-activities", json=data.dict()
        )
        return TrustedActivity.parse_response(response)

    def add_url_path(
        self,
        url: str,
        description: str = None,
    ) -> TrustedActivity:
        """
        Trust browser uploads to only part of a domain by including a specific path.  For example: `github.com/company` will only trust uploads to the `company` repository.

        **Parameters:**

        * **url**: `str` (required) - URL path to trust (ex: `example.com/path`).
        * **description**: `str` - Optional description of the trusted activity.

        **Returns**: A [`TrustedActivity`][trustedactivity-model] object representing
        the newly created trusted activity.
        """

        data = CreateTrustedActivityRequest(
            type=ActivityType.URL_PATH,
            value=url,
            description=description,
            activityActionGroups=[],
        )

        response = self._parent.session.post(
            url="/v2/trusted-activities", json=data.dict()
        )
        return TrustedActivity.parse_response(response)

    def add_slack_workspace(
        self,
        workspace_name: str,
        description: str = None,
    ) -> TrustedActivity:
        """
        Trust activity uploaded through a Slack workspace.

        **Parameters:**

        * **workspace_name**: `str` (required) - Name of the Slack workspace to trust.
        * **description**: `str` - Optional description of the trusted activity.

        **Returns**: A [`TrustedActivity`][trustedactivity-model] object representing
        the newly created trusted activity.
        """

        data = CreateTrustedActivityRequest(
            type=ActivityType.SLACK,
            value=workspace_name,
            description=description,
            activityActionGroups=[],
        )

        response = self._parent.session.post(
            url="/v2/trusted-activities", json=data.dict()
        )
        return TrustedActivity.parse_response(response)

    def add_account_name(
        self,
        account_name: str,
        description: str = None,
        dropbox: bool = False,
        one_drive: bool = False,
    ) -> TrustedActivity:
        """
        Trust activity for a specific corporate account for cloud sync apps installed on user devices.

        **Parameters:**

        * **account_name**: `str` (required) - Account name to trust for the specified cloud sync services.
        * **description**: `str` - Optional description of the trusted activity.
        * **dropbox**: `bool` - Whether to trust Dropbox as a cloud sync service.  Defaults to False.
        * **one_drive** `bool` - Whether to trust OneDrive as a cloud sync service.  Defaults to False.

        At least 1 activity action group (dropbox, one_drive) is required to be trusted.

        **Returns**: A [`TrustedActivity`][trustedactivity-model] object representing
        the newly created trusted activity.
        """

        providers = []

        if dropbox:
            providers.append(ProviderObject(name=Name.DROPBOX))

        if one_drive:
            providers.append(ProviderObject(name=Name.ONE_DRIVE))

        # At least 1 activity action group is required
        if len(providers) < 1:
            raise ValueError(
                "At least 1 cloud sync service (dropbox, one_drive) must be trusted."
            )

        activity_action_group = ActivityActionGroup(
            activityActions=[
                ActivityAction(providers=providers, type=ActivityType.CLOUD_SYNC)
            ],
            name=Name.DEFAULT,
        )

        data = CreateTrustedActivityRequest(
            type=ActivityType.ACCOUNT_NAME,
            value=account_name,
            description=description,
            activityActionGroups=[activity_action_group],
        )

        response = self._parent.session.post(
            url="/v2/trusted-activities", json=data.dict()
        )
        return TrustedActivity.parse_response(response)

    def add_git_repository(
        self,
        git_uri: str,
        description: str = None,
    ) -> TrustedActivity:
        """
        Trust file uploads to a git repository.

        **Parameters:**

        * **git_uri**: `str` (required) - Git URI to trust (ex: `bitbucket.org:exampleent/myrepo`).
        * **description**: `str` - Optional description of the trusted activity.

        **Returns**: A [`TrustedActivity`][trustedactivity-model] object representing
        the newly created trusted activity.
        """

        activity_action_group = ActivityActionGroup(
            activityActions=[ActivityAction(providers=[], type=ActivityType.GIT_PUSH)],
            name=Name.DEFAULT,
        )

        data = CreateTrustedActivityRequest(
            type=ActivityType.GIT_REPOSITORY_URI,
            value=git_uri,
            description=description,
            activityActionGroups=[activity_action_group],
        )

        response = self._parent.session.post(
            url="/v2/trusted-activities", json=data.dict()
        )
        return TrustedActivity.parse_response(response)

    def delete(self, activity_id: int) -> Response:
        """
        Delete a trusted activity.

        **Parameters**:

        * **activity_id** `int` (required) - Unique ID for the trusted activity.

        Usage example:

            >>> client.trusted_activities.v2.delete(23)
            <Response [200]>

        **Returns**: A `requests.Response` indicating success.
        """
        return self._parent.session.delete(f"/v2/trusted-activities/{activity_id}")

    def update(self, trusted_activity: TrustedActivity) -> TrustedActivity:
        """
        Updates a trusted activity.
        The following fields can be updated:

        * `description`: A description of the trusted activity.
        * `value`: The value of the trusted activity, e.g. domain name or Slack workspace name.
        * `type`: The type of the trusted activity.  One of [`ActivityType`][activity-types].
        * `activity_action_group`: The list of actions associated with the activity.

        **Parameters**

        * **trusted_activity**: [`TrustedActivity`][trustedactivity-model] (required) - The modified trusted activity object.

        Usage example:

            >>> activity = client.trusted_activities.get_trusted_activity(2)
            >>> activity.description = "New description"
            >>> client.trusted_activities.v2.update(activity)

        **Returns**: A [`TrustedActivity`][trustedactivity-model] object with updated values from server.
        """

        data = UpdateTrustedActivity(**trusted_activity.dict())
        response = self._parent.session.patch(
            f"/v2/trusted-activities/{trusted_activity.activity_id}",
            json=data.dict(),
        )
        return TrustedActivity.parse_response(response)


class TrustedActivitiesClient:
    def __init__(self, parent):
        self._parent = parent
        self._v2 = None

    @property
    def v2(self):
        if self._v2 is None:
            self._v2 = TrustedActivitiesV2(self._parent)
        return self._v2


class MissingActivityActionGroupsError(IncydrException):
    """An error raised when a trusted activity is missing trusted action groups."""

    def __init__(self):
        super().__init__(
            "Missing Activity Action Group(s) Error: At least 1 action for the domain must be trusted."
        )



from datetime import datetime
from typing import List
from typing import Optional
from typing import Union

from pydantic import Field

from _incydr_sdk.core.models import ResponseModel
from _incydr_sdk.enums import SortDirection
from _incydr_sdk.file_events.models.event import FileEventV2


class SearchFilter(ResponseModel):
    operator: Optional[str] = Field(
        description="The type of match to perform.  Default value is `IS`.",
        example="IS_NOT",
    )
    term: Optional[str] = Field(description="The field to match.", example="user.email")
    value: Optional[Union[List[str], str]] = Field(
        None, description="The input for the search.", example="ari@example.com"
    )


class SearchFilterGroup(ResponseModel):
    filter_clause: Optional[str] = Field(
        alias="filterClause",
        description="Grouping clause for filters.  Default is `AND`.",
        example="AND",
    )
    filters: List[SearchFilter] = Field(
        description="One or more SearchFilters to be combined in a query."
    )


class QueryProblem(ResponseModel):
    """
    A model containing data on a query problem.

    **Fields**:

    * **bad_filter**: `SearchFilter` - The search filter that caused the problem.
    * **description**: `str` - Additional description of the problem.
    * **type**: 'ProblemType' - The type of problem that occurred. Ex: `SEARCH_FAILED`

    """

    bad_filter: Optional[SearchFilter] = Field(
        alias="badFilter",
        description="The search filter that caused this problem.",
    )
    description: Optional[str] = Field(
        description="Additional description of the problem.",
        example="Request timed out.  Refine your filter criteria and try again.",
    )
    type: Optional[str] = Field(
        description="The type of problem that occured.", example="SEARCH_FAILED"
    )


class FileEventsPage(ResponseModel):
    """
    A model representing a page of `FileEventV2` objects.

    **Fields**:

    * **file_events**: `List[FileEventsV2]` - The list of `n` number of file events retrieved from the query, where `n=pg_size`.
    * **next_pg_token**: `str` - The pgToken value from another request to indicate the starting point the next page of results. `nextPgToken` is null if there are no more results or if pgToken was not supplied.
    * **problems**: `List[QueryProblem]` - "List of problems in the request.  A problem with a search request could be an invalid filter value, an operator that can't be used on a term, etc.
    * **total_count**: `int` - Total count of file events found by the query.
    """

    file_events: Optional[List[FileEventV2]] = Field(
        alias="fileEvents", description="List of file events in the response."
    )
    next_pg_token: Optional[str] = Field(
        alias="nextPgToken",
        description="Use as the pgToken value in another request to indicate the starting point for additional page results. nextPgToken is null if there are no more results or if pgToken was not supplied.",
        example="0_147e9445-2f30-4a91-8b2a-9455332e880a_973435567569502913_986467523038446097_163",
    )
    problems: Optional[List[QueryProblem]] = Field(
        description="List of problems in the request.  A problem with a search request could be an invalid filter value, an operator that can't be used on a term, etc.",
    )
    total_count: Optional[int] = Field(
        alias="totalCount",
        description="Total number of file events in the response.",
        example=42,
    )


class SavedSearch(ResponseModel):
    """
    A model representing a saved search.

    **Fields**:

    * **api_version**: `int` - The version of the API used to create the search.
    * **columns**: `List[str]` - The list of columns to be displayed in the web app for the search.
    * **created_by_uid**: `str` - The ID of the user who created the saved search.
    * **created_by_username**: `str` - The username of the user who created the saved search.
    * **creation_timestamp**: `datetime` - The time at which the saved search was created.
    * **group_clause**: `GroupClause` - `AND` or `OR`. Grouping clause for any specified groups. Defaults to `AND`.
    * **groups**: `List[SearchFilterGroup]` - One or more FilterGroups to be combined in a query.
    * **id**: `str` - The ID for the saved search.
    * **modified_by_uid**: `str` - The ID of the user who last modified the saved search.
    * **modified_by_username**: `str` - The username of the user who last modified the saved search.
    * **modified_timestamp**: `datetime` - The time at which the saved search was last modified.
    * **name**: `str` - The name given to the saved search.
    * **notes**: `str` - Optional notes about the search.
    * **srt_dir**: `SortDirection` - `asc` or `desc`. The direction in which to sort the response based on the corresponding key. Defaults to 'asc'.
    * **srt_key**: `str` - One or more values on which the response will be sorted. Defaults to event ID.

    """

    api_version: Optional[int] = Field(
        alias="apiVersion",
        description="Version of the API used to create the search.",
        example=1,
    )
    columns: Optional[List[str]] = Field(
        description="List of columns to be displayed in the web app for the search.",
    )
    created_by_uid: Optional[str] = Field(
        alias="createdByUID",
        description="User UID of the user who created the saved search.",
        example=806150685834341101,
    )
    created_by_username: Optional[str] = Field(
        alias="createdByUsername",
        description="Username of the user who created the saved search.",
        example="adrian@example.com",
    )
    creation_timestamp: Optional[datetime] = Field(
        alias="creationTimestamp",
        description="Time at which the saved search was created.",
        example="2020-10-27T15:16:05.369203Z",
    )
    group_clause: Optional[str] = Field(
        alias="groupClause",
        description="Grouping clause for any specified groups.",
        example="OR",
    )
    groups: Optional[List[SearchFilterGroup]] = Field(
        description="One or more FilterGroups to be combined in a query."
    )
    id: Optional[str] = Field(
        description="Unique identifier for the saved search.",
        example="cde979fa-d551-4be9-b242-39e75b824089",
    )
    modified_by_uid: Optional[str] = Field(
        alias="modifiedByUID",
        description="User UID of the user who last modified the saved search.",
        example=421380797518239242,
    )
    modified_by_username: Optional[str] = Field(
        alias="modifiedByUsername",
        description="Username of the user who last modified the saved search.",
        example="ari@example.com",
    )
    modified_timestamp: Optional[datetime] = Field(
        alias="modifiedTimestamp",
        description="Time at which the saved search was last modified.",
        example="2020-10-27T15:20:26.311894Z",
    )
    name: Optional[str] = Field(
        description="Name given to the saved search.",
        example="Example saved search",
    )
    notes: Optional[str] = Field(
        description="Optional notes about the search.",
        example="This search returns all events.",
    )
    srt_dir: Optional[SortDirection] = Field(
        alias="srtDir", description="Sort direction.", example="asc"
    )
    srt_key: Optional[str] = Field(
        alias="srtKey", description="Search term for sorting.", example="event.id"
    )



from __future__ import annotations

from typing import List
from typing import Optional

from pydantic import Field
from rich.markdown import Markdown

from _incydr_sdk.core.models import Model
from _incydr_sdk.core.models import ResponseModel


class Date(ResponseModel):
    year: Optional[int] = Field(
        None,
        description="Year of date. Must be from 1 to 9999, or 0 if specifying a date without\n a year.",
    )
    month: Optional[int] = Field(
        None,
        description="Month of year. Must be from 1 to 12, or 0 if specifying a year without a\n month and day.",
    )
    day: Optional[int] = Field(
        None,
        description="Day of month. Must be from 1 to 31 and valid for the year and month, or 0\n if specifying a year "
        "by itself or a year and month where the day is not\n significant.",
    )


class RiskProfile(ResponseModel):
    """A model representing a risk profile.

    **Fields**:

    * **active**: `bool` - Whether the actor is active.
    * **cloud_alias**: `List[str]` - List of cloud aliases for the actor.
    * **country**: `str` - The actor's country.
    * **deleted**: `bool` - Whether the actor has been deleted.
    * **department**: `str` - The actor's department.
    * **display_name**: `str` - The actor's display name.
    * **division**: `str` - The actor's division.
    * **employment_type**: `str` - The actor's employment type.
    * **end_date**: `Date` - Departure date for the actor
    * **locality**: `str` - The actor's locality (city).
    * **manager_display_name**: `str` - The actor's manager's display name.
    * **manager_id**: `str` - Unique actor ID of the actor's manager.
    * **manager_username**: `str` - The Code42 actorname of the actor's manager.
    * **notes**: `str` - Additional notes about the actor.
    * **region**: `str` - The actor's region (state).
    * **start_date**: `Date` - Starting date for the actor.
    * **support_user**: `bool` - Whether the actor is a support actor.
    * **tenant_id**: `str` - Unique tenant ID.
    * **title**: `str` - The actor's job title.
    * **user_id**: `str` - Unique actor ID.
    * **username**: `str` - actor's Code42 actorname.
    """

    active: Optional[bool] = Field(None, description="Whether the actor is active.")
    cloud_aliases: Optional[List[str]] = Field(
        None, description="A list of cloud aliases for the actor.", alias="cloudAliases"
    )
    country: Optional[str] = Field(None, description="The actor's country.")
    deleted: Optional[bool] = Field(None, description="Whether the actor is deleted.")
    department: Optional[str] = Field(None, description="The actor's department.")
    display_name: Optional[str] = Field(
        None, description="The actor's friendly display name.", alias="displayName"
    )
    division: Optional[str] = Field(None, description="The actor's division.")
    employment_type: Optional[str] = Field(
        None, description="The actor's employment type.", alias="employmentType"
    )
    end_date: Optional[Date] = Field(None, alias="endDate")
    locality: Optional[str] = Field(None, description="The actor's locality (city).")
    manager_display_name: Optional[str] = Field(
        None,
        description="The actor's manager's friendly display name.",
        alias="managerDisplayName",
    )
    manager_id: Optional[str] = Field(
        None,
        description="The Code42 actor ID of the actor's manager.",
        alias="managerId",
    )
    manager_username: Optional[str] = Field(
        None,
        description="The Code42 actorname of the actor's manager.",
        alias="managerUsername",
    )
    notes: Optional[str] = Field(
        None,
        description="Notes about the actor.",
        table=lambda f: f if f is None else Markdown(f),
    )
    region: Optional[str] = Field(None, description="The actor's region (state).")
    start_date: Optional[Date] = Field(None, alias="startDate")
    support_user: Optional[bool] = Field(
        None, description="Whether the actor is a support actor.", alias="supportUser"
    )
    tenant_id: Optional[str] = Field(
        None, description="A unique tenant ID.", alias="tenantId"
    )
    title: Optional[str] = Field(None, description="The actor's job title.")
    user_id: Optional[str] = Field(
        None, description="A unique actor ID.", alias="userId"
    )
    username: Optional[str] = Field(None, description="The actor's Code42 actorname.")


class RiskProfilesPage(ResponseModel):
    """
    A model representing a page of `RiskProfile` objects.

    **Fields**:

    * **total_count**: `int` The total count of all risk profiles.
    * **user_risk_profiles**: `List[RiskProfile]` The list of `n` number of risk profiles
                            retrieved from the query, where `n=page_size`.
    """

    total_count: Optional[int] = Field(
        None,
        description="The total count of all risk profiles.",
        example=10,
        alias="totalCount",
    )
    user_risk_profiles: Optional[List[RiskProfile]] = Field(
        None, alias="userRiskProfiles"
    )


class QueryRiskProfilesRequest(Model):
    page: Optional[int]
    page_size: Optional[int]
    manager_id: Optional[str]
    title: Optional[str]
    division: Optional[str]
    department: Optional[str]
    employment_type: Optional[str]
    country: Optional[str]
    region: Optional[str]
    locality: Optional[str]
    active: Optional[bool]
    deleted: Optional[bool]
    support_user: Optional[bool]

    class Config:
        use_enum_values = True


class UpdateRiskProfileRequest(Model):
    endDate: Optional[Date] = None
    notes: Optional[str] = Field(
        None,
        description="Notes to add to the risk profile.",
        example="These are my notes",
    )
    startDate: Optional[Date] = None


class AddCloudAliasesRequest(Model):
    cloudAliases: Optional[List[str]] = None
    userId: Optional[str] = Field(
        None, description="The ID of the actor to add cloud aliases.", example="123"
    )


class DeleteCloudAliasesRequest(Model):
    cloudAliases: Optional[List[str]] = None
    userId: Optional[str] = Field(
        None, description="The ID of the actor to delete cloud aliases.", example="123"
    )



import logging
import socket
import ssl
import sys
from logging.handlers import SysLogHandler

from _incydr_cli.logger import ServerProtocol


class SyslogServerNetworkConnectionError(Exception):
    """An error raised when the connection is disrupted during logging."""

    def __init__(self):
        super().__init__(
            "The network connection broke while sending results. "
            "This might happen if your connection requires TLS and you are attempting "
            "unencrypted TCP communication."
        )


class NoPrioritySysLogHandler(SysLogHandler):
    """
    Overrides the default implementation of SysLogHandler to not send a `<PRI>` at the
    beginning of the message. Most CEF consumers seem to not expect the `<PRI>` to be
    present in CEF messages. Attach to a logger via `.addHandler` to use.

    `self.socket` is lazily loaded for testing purposes, so the connection does not get
    made for TCP/TLS until the first log record is about to be transmitted.

    Args:
        hostname: The hostname of the syslog server to send log messages to.
        port: The port of the syslog server to send log messages to.
        protocol: The protocol over which to submit syslog messages. Accepts TCP, UDP, or TLS.
        certs: Certs to specify when using TLS-TCP for the `protocol` argument. Use "ignore" for
            ssl.CERT_NONE (ignoring certificate validation).
    """

    def __init__(self, hostname, port, protocol, certs):
        self._hostname = hostname
        self._port = port
        self._protocol = protocol
        self._certs = certs
        self.address = (hostname, port)
        logging.Handler.__init__(self)
        self.socktype = _try_get_socket_type_from_protocol(protocol)
        self.socket = None

    @property
    def _wrap_socket(self):
        return self._protocol == ServerProtocol.TLS_TCP

    def connect_socket(self):
        """Call to initialize the socket. If using TCP/TLS, it will also establish the connection."""
        if not self.socket:
            self.socket = self._create_socket(self._hostname, self._port, self._certs)

    def _create_socket(self, hostname, port, certs):
        try:
            socket_info = socket.getaddrinfo(hostname, port, 0, self.socktype)[0]
        except KeyError:
            raise OSError("getaddrinfo() returns an empty list")
        address_family, sock_type, proto, _, sa = socket_info
        sock = None
        try:
            sock = socket.socket(address_family, sock_type, proto)
            if self._wrap_socket:
                sock = _wrap_socket_for_ssl(sock, certs, hostname)
            if sock_type == socket.SOCK_STREAM:
                sock = _connect_socket(sock, sa)
            return sock
        except Exception as exc:
            if sock is not None:
                sock.close()
            raise exc

    def emit(self, record):
        try:
            self._send_record(record)
        except Exception:
            self.handleError(record)

    def handleError(self, record):
        """Override logger's `handleError` method to exit if an exception is raised while trying to
        log, otherwise it would continue to gather and process events if the connection breaks but send
        them nowhere.
        """
        t, _, _ = sys.exc_info()
        if issubclass(t, ConnectionError):
            raise SyslogServerNetworkConnectionError()
        super().handleError(record)

    def _send_record(self, record):
        formatted_record = self.format(record)
        msg = formatted_record + "\n"
        msg = msg.encode("utf-8")
        if self.socktype == socket.SOCK_DGRAM:
            self.socket.sendto(msg, self.address)
        else:
            self.socket.sendall(msg)

    def close(self):
        if self._wrap_socket:
            self.socket.unwrap()
        self.socket.close()
        logging.Handler.close(self)


def _wrap_socket_for_ssl(sock, certs, hostname):
    do_ignore_certs = certs and certs.lower() == "ignore"
    if do_ignore_certs:
        certs = None
    context = ssl.create_default_context(cafile=certs)
    if do_ignore_certs:
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
    return context.wrap_socket(sock, server_hostname=hostname)


def _connect_socket(sock, sa):
    sock.settimeout(10)
    sock.connect(sa)
    # Set timeout back to None for 'blocking' mode, required for `sendall()`.
    sock.settimeout(None)
    return sock


def _try_get_socket_type_from_protocol(protocol):
    socket_type = _get_socket_type_from_protocol(protocol)
    if socket_type is None:
        raise ValueError(
            "Could not determine socket type. "
            f"Expected one of {list(ServerProtocol)}, but got {protocol}."
        )
    return socket_type


def _get_socket_type_from_protocol(protocol):
    if protocol in [ServerProtocol.TCP, ServerProtocol.TLS_TCP]:
        return socket.SOCK_STREAM
    elif protocol == ServerProtocol.UDP:
        return socket.SOCK_DGRAM



from itertools import count
from typing import Iterator
from typing import List
from typing import Union

from _incydr_sdk.enums.watchlists import WatchlistType
from _incydr_sdk.exceptions import WatchlistNotFoundError
from _incydr_sdk.watchlists.clientv1 import WatchlistsV1
from _incydr_sdk.watchlists.models.requests import CreateWatchlistRequest
from _incydr_sdk.watchlists.models.requests import ListWatchlistsRequestV2
from _incydr_sdk.watchlists.models.requests import UpdateExcludedActorsRequest
from _incydr_sdk.watchlists.models.requests import UpdateIncludedActorsRequest
from _incydr_sdk.watchlists.models.requests import UpdateIncludedDepartmentsRequest
from _incydr_sdk.watchlists.models.requests import UpdateIncludedDirectoryGroupsRequest
from _incydr_sdk.watchlists.models.requests import UpdateWatchlistRequest
from _incydr_sdk.watchlists.models.responses import ExcludedActorsList
from _incydr_sdk.watchlists.models.responses import IncludedActorsList
from _incydr_sdk.watchlists.models.responses import IncludedDepartment
from _incydr_sdk.watchlists.models.responses import IncludedDepartmentsList
from _incydr_sdk.watchlists.models.responses import IncludedDirectoryGroup
from _incydr_sdk.watchlists.models.responses import IncludedDirectoryGroupsList
from _incydr_sdk.watchlists.models.responses import Watchlist
from _incydr_sdk.watchlists.models.responses import WatchlistActor
from _incydr_sdk.watchlists.models.responses import WatchlistMembersListV2
from _incydr_sdk.watchlists.models.responses import WatchlistsPage


class WatchlistsClient:
    def __init__(self, parent):
        self._parent = parent
        self._v1 = None
        self._v2 = None

    @property
    def v1(self):
        if self._v1 is None:
            self._v1 = WatchlistsV1(self._parent)
        return self._v1

    @property
    def v2(self):
        if self._v2 is None:
            self._v2 = WatchlistsV2(self._parent)
        return self._v2


class WatchlistsV2:
    """
    Client for `/v2/watchlists` endpoints.

    Usage example:

        >>> import incydr
        >>> client = incydr.Client(**kwargs)
        >>> client.watchlists.v2.get_page()
    """

    def __init__(self, parent):
        self._parent = parent
        self._watchlist_type_id_map = {}
        self._uri = "/v2/watchlists"

    def get_page(
        self, page_num: int = 1, page_size: int = None, actor_id: str = None
    ) -> WatchlistsPage:
        """
        Get a page of watchlists.

        Filter results by passing appropriate parameters:

        **Parameters**:

        * **page_num**: `int` - Page number for results, starting at 1.
        * **page_size**: `int` - Max number of results to return for a page.
        * **actor_id**: `str` - Matches watchlists where the actor is a member.

        **Returns**: A [`WatchlistsPage`][watchlistspage-model] object.
        """
        page_size = page_size or self._parent.settings.page_size
        data = ListWatchlistsRequestV2(
            page=page_num, pageSize=page_size, actorId=actor_id
        )
        response = self._parent.session.get(self._uri, params=data.dict())
        return WatchlistsPage.parse_response(response)

    def iter_all(
        self, page_size: int = None, actor_id: str = None
    ) -> Iterator[Watchlist]:
        """
        Iterate over all watchlists.

        Accepts the same parameters as `.get_page()` excepting `page_num`.

        **Returns**: A generator yielding individual [`Watchlist`][watchlist-model] objects.
        """
        page_size = page_size or self._parent.settings.page_size
        for page_num in count(1):
            page = self.get_page(
                page_num=page_num, page_size=page_size, actor_id=actor_id
            )
            yield from page.watchlists
            if len(page.watchlists) < page_size:
                break

    def get(self, watchlist_id: str) -> Watchlist:
        """
        Get a single watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.

        **Returns**: A [`Watchlist`][watchlist-model] object.
        """
        response = self._parent.session.get(f"{self._uri}/{watchlist_id}")
        return Watchlist.parse_response(response)

    def create(
        self, watchlist_type: WatchlistType, title: str = None, description: str = None
    ) -> Watchlist:
        """
        Create a new watchlist.

        **Parameters**:

        * **watchlist_type**: [`WatchlistType`][watchlist-types] (required) - Type of the watchlist to create.
        * **title**: The required title for a custom watchlist.
        * **description**: The optional description for a custom watchlist.

        **Returns**: A ['Watchlist`][watchlist-model] object.
        """
        if watchlist_type == "CUSTOM":
            if title is None:
                raise ValueError("`title` value is required for custom watchlists.")

        data = CreateWatchlistRequest(
            description=description, title=title, watchlistType=watchlist_type
        )
        response = self._parent.session.post(url=self._uri, json=data.dict())
        watchlist = Watchlist.parse_response(response)
        self._watchlist_type_id_map[watchlist_type] = watchlist.watchlist_id
        return watchlist

    def delete(self, watchlist_id: str):
        """
        Delete a watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.

        **Returns**: A `requests.Response` indicating success.
        """
        return self._parent.session.delete(f"{self._uri}/{watchlist_id}")

    def update(
        self, watchlist_id: str, title: str = None, description: str = None
    ) -> Watchlist:
        """
        Update a custom watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.
        * **title**: `str` - Updated title for a custom watchlist.  Defaults to None.
        * **description: `str` - Updated description for a custom watchlist.  Pass an empty string to clear this field.  Defaults to None.

        **Returns**: A [`Watchlist`][watchlist-model] object.
        """
        paths = []
        if title:
            paths += ["title"]
        if description:
            paths += ["description"]
        query = {"paths": paths}
        data = UpdateWatchlistRequest(description=description, title=title)
        response = self._parent.session.patch(
            f"{self._uri}/{watchlist_id}", params=query, json=data.dict()
        )
        return Watchlist.parse_response(response)

    def get_member(self, watchlist_id: str, actor_id: str) -> WatchlistActor:
        """
        Get a single member of a watchlist. A member may have been added as an included actor, or is a member of an included department, etc.

        **Parameters**:

         * **watchlist_id**: `str` (required) - Watchlist ID.
        * **actor_id**: `str` (required) - Unique actor ID.

        **Returns**: A [`WatchlistActor`][watchlistactor-model] object.
        """
        response = self._parent.session.get(
            f"{self._uri}/{watchlist_id}/members/{actor_id}"
        )
        return WatchlistActor.parse_response(response)

    def list_members(
        self,
        watchlist_id: Union[str, WatchlistType],
        page_num: int = None,
        page_size: int = None,
    ) -> WatchlistMembersListV2:
        """
        Get a list of all members of a watchlist. These actors may have been added as an included actor, or are members of an included department, etc.

        **Parameters**:

        * **watchlist_id**: `str`(required) - Watchlist ID.
        * **page_num**: `int` (optional) - The page number to fetch, starting at 1.
        * **page_size**: `int` (optional) - Max number of results to return for a page.

        **Returns**: A [`WatchlistMembersListV2`][watchlistmemberslistv2-model] object.
        """
        page_size = page_size or self._parent.settings.page_size
        response = self._parent.session.get(
            f"{self._uri}/{watchlist_id}/members",
            params={"page": page_num, "page_size": page_size},
        )
        return WatchlistMembersListV2.parse_response(response)

    def iter_all_members(
        self, watchlist_id: Union[str, WatchlistType], page_size: int = None
    ) -> Iterator[WatchlistActor]:
        """
        Iterate over all members of a watchlist. These actors may have been added as an included actor, or are members of an included department, etc.

        Accepts the same parameters as `.list_members()` excepting `page_num`.

        **Returns**: A generator that yields [`WatchlistActor`][watchlistactor-model] objects.
        """
        page_size = page_size or self._parent.settings.page_size
        for page_num in count(1):
            page = self.list_members(
                watchlist_id=watchlist_id, page_size=page_size, page_num=page_num
            )
            yield from page.watchlist_members
            if len(page.watchlist_members) < page_size:
                break

    def add_included_actors(self, watchlist_id: str, actor_ids: Union[str, List[str]]):
        """
        Include individual actors on a watchlist.

        **Parameters**:

        * **watchlist**: `str` (required) - Watchlist ID.
        * **actor_ids**: `str`, `List[str]` (required) - List of unique actor IDs to include on the watchlist. A maximum
            of 100 actors can be processed in a single request.

        **Returns**: A `requests.Response` indicating success.
        """
        data = UpdateIncludedActorsRequest(
            actorIds=actor_ids if isinstance(actor_ids, List) else [actor_ids],
            watchlistId=watchlist_id,
        )
        return self._parent.session.post(
            url=f"{self._uri}/{watchlist_id}/included-actors/add", json=data.dict()
        )

    def remove_included_actors(
        self, watchlist_id: str, actor_ids: Union[str, List[str]]
    ):
        """
        Remove included actors from a watchlist.

        **Parameters**:

        * **watchlist**: `str` (required) - Watchlist ID.
        * **actor_ids**: `str`, `List[str]` (required) - List of unique actor IDs to remove from the watchlist. A maximum
            of 100 actors can be processed in a single request.

        **Returns**: A `requests.Response` indicating success.
        """

        data = UpdateIncludedActorsRequest(
            actorIds=actor_ids if isinstance(actor_ids, List) else [actor_ids],
            watchlistId=watchlist_id,
        )
        return self._parent.session.post(
            url=f"{self._uri}/{watchlist_id}/included-actors/delete", json=data.dict()
        )

    def get_included_actor(self, watchlist_id: str, actor_id: str) -> WatchlistActor:
        """
        Get an included actor from a watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.
        * **actor_id**: `str` (required) - Unique actor ID.

        **Returns**: A [`WatchlistActor`][watchlistactor-model] object.
        """
        response = self._parent.session.get(
            url=f"{self._uri}/{watchlist_id}/included-actors/{actor_id}"
        )
        return WatchlistActor.parse_response(response)

    def list_included_actors(
        self, watchlist_id: str, page_num: int = None, page_size: int = None
    ) -> IncludedActorsList:
        """
        List individual actors included on a watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.
        * **page_num**: `int` (optional) - The page number to fetch, starting at 1.
        * **page_size**: `int` (optional) - Max number of results to return for a page.

        **Returns**: An [`IncludedActorsList`][includedactorslist-model] object.
        """
        page_size = page_size or self._parent.settings.page_size
        response = self._parent.session.get(
            url=f"{self._uri}/{watchlist_id}/included-actors",
            params={"page": page_num, "page_size": page_size},
        )
        return IncludedActorsList.parse_response(response)

    def iter_all_included_actors(
        self, watchlist_id: str, page_size: int = None
    ) -> Iterator[WatchlistActor]:
        """
        Iterate over all individual actors included on a watchlist.

        Accepts the same parameters as `.list_included_actors()` excepting `page_num`.

        **Returns**: A generator yielding individual [`WatchlistActor`][watchlistactor-model] objects.
        """
        page_size = page_size or self._parent.settings.page_size
        for page_num in count(1):
            page = self.list_included_actors(
                watchlist_id=watchlist_id, page_num=page_num, page_size=page_size
            )
            yield from page.included_actors
            if len(page.included_actors) < page_size:
                break

    def add_excluded_actors(self, watchlist_id: str, actor_ids: Union[str, List[str]]):
        """
        Exclude individual actors from a watchlist.

        **Parameters**:

        * **watchlist**: `str` (required) - Watchlist ID.
        * **actor_ids**: `str`, `List[str]` (required) - List of unique actor IDs to exclude from the watchlist. A maximum
            of 100 actors can be processed in a single request.

        **Returns**: A `requests.Response` indicating success.
        """
        data = UpdateExcludedActorsRequest(
            actorIds=actor_ids if isinstance(actor_ids, List) else [actor_ids],
        )
        return self._parent.session.post(
            url=f"{self._uri}/{watchlist_id}/excluded-actors/add", json=data.dict()
        )

    def remove_excluded_actors(
        self, watchlist_id: str, actor_ids: Union[str, List[str]]
    ):
        """
        Remove excluded actors from a watchlist.

        **Parameters**:

        * **watchlist**: `str` (required) - Watchlist ID.
        * **actor_ids**: `str`, `List[str]` (required) - List of unique actor IDs to remove from the exclusion list. A
            maximum of 100 actors can be processed in a single request.

        **Returns**: A `requests.Response` indicating success.
        """
        data = UpdateExcludedActorsRequest(
            actorIds=actor_ids if isinstance(actor_ids, List) else [actor_ids],
        )
        return self._parent.session.post(
            url=f"{self._uri}/{watchlist_id}/excluded-actors/delete", json=data.dict()
        )

    def list_excluded_actors(
        self, watchlist_id: str, page_num: int = None, page_size: int = None
    ) -> ExcludedActorsList:
        """
        List individual actors excluded from a watchlist.

        * **watchlist_id**: `str` (required) - Watchlist ID.
        * **page_num**: `int` (optional) - The page number to fetch, starting at 1.
        * **page_size**: `int` (optional) - Max number of results to return for a page.


        **Returns**: An [`ExcludedActorsList`][excludedactorslist-model] object.
        """
        page_size = page_size or self._parent.settings.page_size
        response = self._parent.session.get(
            f"{self._uri}/{watchlist_id}/excluded-actors",
            params={"page": page_num, "page_size": page_size},
        )
        return ExcludedActorsList.parse_response(response)

    def iter_all_excluded_actors(
        self, watchlist_id: str, page_size: int = None
    ) -> Iterator[WatchlistActor]:
        """
        Iterate over all individual actors excluded from a watchlist.

        Accepts the same parameters as `.list_excluded_actors()` excepting `page_num`.

        **Returns**: A generator yielding individual [`WatchlistActor`][watchlistactor-model] objects.
        """
        page_size = page_size or self._parent.settings.page_size
        for page_num in count(1):
            page = self.list_excluded_actors(
                page_num=page_num, page_size=page_size, watchlist_id=watchlist_id
            )
            yield from page.excluded_actors
            if len(page.excluded_actors) < page_size:
                break

    def get_excluded_actor(self, watchlist_id: str, actor_id: str) -> WatchlistActor:
        """
        Get an excluded actor from a watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.
        * **actor_id**: `str` (required) - Unique actor ID.

        **Returns**: A [`WatchlistActor`][watchlistactor-model] object.
        """
        response = self._parent.session.get(
            f"{self._uri}/{watchlist_id}/excluded-actors/{actor_id}"
        )
        return WatchlistActor.parse_response(response)

    def add_directory_groups(self, watchlist_id: str, group_ids: Union[str, List[str]]):
        """
        Include directory groups on a watchlist. Use the `directory_groups` client to see available directory groups.

        **Parameters**:

        * **watchlist**: `str` (required) - Watchlist ID.
        * **group_ids**: `str`, `List[str]` (required) - List of directory group IDs to include on the watchlist.

        **Returns**: A `requests.Response` indicating success.
        """
        data = UpdateIncludedDirectoryGroupsRequest(
            groupIds=group_ids if isinstance(group_ids, List) else [group_ids]
        )
        return self._parent.session.post(
            url=f"{self._uri}/{watchlist_id}/included-directory-groups/add",
            json=data.dict(),
        )

    def remove_directory_groups(
        self, watchlist_id: str, group_ids: Union[str, List[str]]
    ):
        """
        Remove included directory groups from a watchlist.

        **Parameters**:

        * **watchlist**: `str` (required) - Watchlist ID.
        * **group_ids**: `str`, `List[str]` (required) - List of directory group IDs to remove from the watchlist.

        **Returns**: A `requests.Response` indicating success.
        """
        data = UpdateIncludedDirectoryGroupsRequest(
            groupIds=group_ids if isinstance(group_ids, List) else [group_ids]
        )
        return self._parent.session.post(
            url=f"{self._uri}/{watchlist_id}/included-directory-groups/delete",
            json=data.dict(),
        )

    def list_directory_groups(
        self, watchlist_id: str, page_num: int = None, page_size: int = None
    ) -> IncludedDirectoryGroupsList:
        """
        List directory groups included on a watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.
        * **page_num**: `int` (optional) - The page number to fetch, starting at 1.
        * **page_size**: `int` (optional) - Max number of results to return for a page.

        **Returns**: An [`IncludedDirectoryGroupsList`][includeddirectorygroupslist-model] object.
        """
        page_size = page_size or self._parent.settings.page_size
        response = self._parent.session.get(
            f"{self._uri}/{watchlist_id}/included-directory-groups",
            params={"page": page_num, "page_size": page_size},
        )
        return IncludedDirectoryGroupsList.parse_response(response)

    def iter_all_directory_groups(
        self, watchlist_id: str, page_size: int = None
    ) -> Iterator[IncludedDirectoryGroup]:
        """
        Iterate over all directory groups included on a watchlist.

        Accepts the same parameters as `.list_directory_groups()` excepting `page_num`.

        **Returns**: A generator yielding individual [`IncludedDirectoryGroup`][includeddirectorygroup-model] objects.
        """
        page_size = page_size or self._parent.settings.page_size
        for page_num in count(1):
            page = self.list_directory_groups(
                page_num=page_num, page_size=page_size, watchlist_id=watchlist_id
            )
            yield from page.included_directory_groups
            if len(page.included_directory_groups) < page_size:
                break

    def get_directory_group(
        self, watchlist_id: str, group_id: str
    ) -> IncludedDirectoryGroup:
        """
        Get an included directory group from a watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.
        * **group_id**: `str` (required) - Directory group ID.

        **Returns**: An [`IncludedDirectoryGroup`][includeddirectorygroup-model] object.
        """

        response = self._parent.session.get(
            f"{self._uri}/{watchlist_id}/included-directory-groups/{group_id}"
        )
        return IncludedDirectoryGroup.parse_response(response)

    def add_departments(
        self,
        watchlist_id: str,
        departments: Union[str, List[str]],
    ):
        """
        Include departments on a watchlist. Use the `departments` client to see available departments.

        **Parameters**:

        * **watchlist**: `str` (required) - Watchlist ID.
        * **departments**: `str`, `List[str]` (required) - List of departments to include on the watchlist.

        **Returns**: A `requests.Response` indicating success.
        """
        data = UpdateIncludedDepartmentsRequest(
            departments=departments if isinstance(departments, List) else [departments]
        )
        return self._parent.session.post(
            url=f"{self._uri}/{watchlist_id}/included-departments/add", json=data.dict()
        )

    def remove_departments(
        self,
        watchlist_id: str,
        departments: Union[str, List[str]],
    ):
        """
        Remove included departments from a watchlist.

        **Parameters**:

        * **watchlist**: `str` - Watchlist ID or a watchlist type.  An ID must be provided for custom watchlists.
        * **departments**: `str`, `List[str]` (required) - List of departments to remove from the watchlist.

        **Returns**: A `requests.Response` indicating success.
        """
        data = UpdateIncludedDepartmentsRequest(
            departments=departments if isinstance(departments, List) else [departments]
        )
        return self._parent.session.post(
            url=f"{self._uri}/{watchlist_id}/included-departments/delete",
            json=data.dict(),
        )

    def list_departments(
        self, watchlist_id: str, page_num: int = None, page_size: int = None
    ) -> IncludedDepartmentsList:
        """
        List departments included on a watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.
        * **page_num**: `int` - Page number for results, starting at 1.
        * **page_size**: `int` - Max number of results to return for a page.

        **Returns**: An [`IncludedDepartmentsList`][includeddepartmentslist-model] object.
        """
        response = self._parent.session.get(
            f"{self._uri}/{watchlist_id}/included-departments",
            params={"page": page_num, "page_size": page_size},
        )
        return IncludedDepartmentsList.parse_response(response)

    def iter_all_departments(
        self, watchlist_id: str, page_size: int = None
    ) -> Iterator[IncludedDepartment]:
        """
        Iterate over all departments included on a watchlist.

        Accepts the same parameters as `.list_departments()` excepting `page_num`.

        **Returns**: A generator yielding individual [`IncludedDepartment`][includeddepartment-model] objects.

        """
        page_size = page_size or self._parent.settings.page_size
        for page_num in count(1):
            page = self.list_departments(
                page_num=page_num, page_size=page_size, watchlist_id=watchlist_id
            )
            yield from page.included_departments
            if len(page.included_departments) < page_size:
                break

    def get_department(self, watchlist_id: str, department: str) -> IncludedDepartment:
        """
        Get an included department from a watchlist.

        **Parameters**:

        * **watchlist_id**: `str` (required) - Watchlist ID.
        * **department**: `str` (required) - A included department.

        **Returns**: An [`IncludedDepartment`][includeddepartment-model] object.
        """
        response = self._parent.session.get(
            f"{self._uri}/{watchlist_id}/included-departments/{department}"
        )
        return IncludedDepartment.parse_response(response)

    def get_id_by_name(self, name: Union[str, WatchlistType]):
        """
        Get a watchlist ID by either its type (ex: `DEPARTING_EMPLOYEE`) or its title in the case of `CUSTOM` watchlists.

        **Parameters**:

        * **name**: `str`, [`WatchlistType`][watchlist-types] (required) - A `WatchlistType` or in the case of `CUSTOM` watchlists, the watchlist `title`.

        **Returns**: A watchlist ID (`str`).
        """

        def _lookup_ids(self):
            """Map watchlist types to IDs, if they exist."""
            self._watchlist_type_id_map = {}
            watchlists = self.get_page(page_size=100).watchlists
            for item in watchlists:
                if item.list_type == "CUSTOM":
                    # store title for custom lists instead of list_type
                    self._watchlist_type_id_map[item.title] = item.watchlist_id
                self._watchlist_type_id_map[item.list_type] = item.watchlist_id

        watchlist_id = self._watchlist_type_id_map.get(name)
        if not watchlist_id:
            # if not found, reset ID cache
            _lookup_ids(self)
            watchlist_id = self._watchlist_type_id_map.get(name)
            if not watchlist_id:
                raise WatchlistNotFoundError(name)
        return watchlist_id



from datetime import datetime
from typing import List
from typing import Optional

from pydantic import Field

from _incydr_sdk.core.models import ResponseModel


class CreatorUser(ResponseModel):
    user_id: Optional[str] = Field(
        None,
        alias="userId",
    )
    username: Optional[str] = Field(
        None,
    )


class CreatorPrincipal(ResponseModel):
    type: Optional[str] = Field(
        None,
    )
    principal_id: Optional[str] = Field(
        None,
        alias="principalId",
    )
    display_name: Optional[str] = Field(
        None,
        alias="displayName",
    )


class Matter(ResponseModel):
    """A model representing a legal hold matter.

    **Fields**:

    * **matter_id**: `str`
    * **name**: `str`
    * **description**: `str`
    * **notes**: `str`
    * **active**: `bool`
    * **creation_date**: `datetime`
    * **policy_id**: `str`
    * **creator**: `CreatorUser`
    * **creator_principal**: `CreatorPrincipal`
    """

    matter_id: Optional[str] = Field(
        None,
        alias="matterId",
    )
    name: Optional[str] = Field(
        None,
    )
    description: Optional[str] = Field(
        None,
    )
    notes: Optional[str] = Field(
        None,
    )
    active: Optional[bool] = Field(
        None,
    )
    creation_date: Optional[datetime] = Field(
        None,
        alias="creationDate",
    )
    policy_id: Optional[str] = Field(
        None,
        alias="policyId",
    )
    creator: Optional[CreatorUser] = Field(
        None,
    )
    creator_principal: Optional[CreatorPrincipal] = Field(
        None,
        alias="creatorPrincipal",
    )


class MattersPage(ResponseModel):
    matters: Optional[List[Matter]] = Field(
        None,
    )


class CustodianMatter(ResponseModel):
    membership_active: Optional[bool] = Field(
        None,
        alias="membershipActive",
    )
    membership_creation_date: Optional[datetime] = Field(
        None,
        alias="membershipCreationDate",
    )
    matter_id: Optional[str] = Field(
        None,
        alias="matterId",
    )
    name: Optional[str] = Field(
        None,
    )


class CustodianMattersPage(ResponseModel):
    matters: Optional[List[CustodianMatter]] = Field(
        None,
    )


class Custodian(ResponseModel):
    membership_active: Optional[bool] = Field(
        None,
        alias="membershipActive",
    )
    membership_creation_date: Optional[datetime] = Field(
        None,
        alias="membershipCreationDate",
    )
    user_id: Optional[str] = Field(
        None,
        alias="userId",
    )
    name: Optional[str] = Field(
        None,
    )
    email: Optional[str] = Field(
        None,
    )


class CustodiansPage(ResponseModel):
    custodians: Optional[List[Custodian]] = Field(
        None,
    )


class AddCustodianResponseMatter(ResponseModel):
    matter_id: Optional[str] = Field(None, alias="matterId")
    name: Optional[str] = Field(
        None,
    )


class AddCustodianResponseCustodian(ResponseModel):
    user_id: Optional[str] = Field(None, alias="userId")
    username: Optional[str] = Field(
        None,
    )
    email: Optional[str] = Field(
        None,
    )


class AddCustodianResponse(ResponseModel):
    membership_active: Optional[bool] = Field(None, alias="membershipActive")
    membership_creation_date: Optional[datetime] = Field(
        None, alias="membershipCreationDate"
    )
    matter: Optional[AddCustodianResponseMatter] = Field(
        None,
    )
    custodian: Optional[AddCustodianResponseCustodian] = Field(
        None,
    )


class LegalHoldPolicy(ResponseModel):
    policy_id: Optional[str] = Field(None, alias="policyId")
    name: Optional[str] = Field(None)
    creation_date: Optional[datetime] = Field(None, alias="creationDate")
    modification_date: Optional[datetime] = Field(None, alias="modificationDate")
    creator_user: Optional[CreatorUser] = Field(None, alias="creatorUser")
    creator_principal: Optional[CreatorPrincipal] = Field(
        None,
        alias="creatorPrincipal",
    )


class LegalHoldPolicyPage(ResponseModel):
    policies: Optional[List[LegalHoldPolicy]]
